@@ -18,6 +18,7 @@ __all__ = [
     # Core API
     "App",
     "Cell",
+    "AppMeta",
     "create_asgi_app",
     "MarimoIslandGenerator",
     "MarimoStopError",
@@ -60,6 +61,7 @@ __all__ = [
     "nav_menu",
     "notebook_dir",
     "notebook_location",
+    "outline",
     "output",
     "pdf",
     "persistent_cache",
@@ -98,6 +100,7 @@ from marimo._output.formatting import as_html, iframe, plain
 from marimo._output.hypertext import Html
 from marimo._output.justify import center, left, right
 from marimo._output.md import latex, md
+from marimo._output.outline import outline
 from marimo._output.show_code import show_code
 from marimo._plugins import ui
 from marimo._plugins.stateless import mpl, status
@@ -125,6 +128,7 @@ from marimo._plugins.stateless.tabs import tabs
 from marimo._plugins.stateless.tree import tree
 from marimo._plugins.stateless.video import video
 from marimo._runtime import output
+from marimo._runtime.app_meta import AppMeta
 from marimo._runtime.capture import (
     capture_stderr,
     capture_stdout,
