@@ -6,9 +6,16 @@ from enum import Enum
 from typing import TYPE_CHECKING, Any, Optional
 
 from marimo._ai._tools.base import ToolBase
-from marimo._ai._tools.types import SuccessResult
+from marimo._ai._tools.types import (
+    MarimoCellConsoleOutputs,
+    MarimoErrorDetail,
+    SuccessResult,
+    ToolGuidelines,
+)
 from marimo._ai._tools.utils.exceptions import ToolExecutionError
 from marimo._ast.models import CellData
+from marimo._messaging.errors import Error
+from marimo._messaging.ops import CellOp, VariableValue
 from marimo._types.ids import CellId_t, SessionId
 
 if TYPE_CHECKING:
@@ -45,19 +52,6 @@ class GetLightweightCellMapOutput(SuccessResult):
     preview_lines: int = 3
 
 
-@dataclass
-class ErrorDetail:
-    type: str
-    message: str
-    traceback: list[str]
-
-
-@dataclass
-class CellErrors:
-    has_errors: bool
-    error_details: Optional[list[ErrorDetail]]
-
-
 @dataclass
 class CellRuntimeMetadata:
     # String form of the runtime state (see marimo._ast.cell.RuntimeStateType);
@@ -67,16 +61,7 @@ class CellRuntimeMetadata:
     execution_time: Optional[float] = None
 
 
-@dataclass
-class CellVariableValue:
-    name: str
-    # Cell variables can be arbitrary Python values (int, str, list, dict, ...),
-    # so we keep this as Any to reflect actual runtime.
-    value: Optional[Any] = None
-    data_type: Optional[str] = None
-
-
-CellVariables = dict[str, CellVariableValue]
+CellVariables = dict[str, VariableValue]
 
 
 @dataclass
@@ -84,7 +69,7 @@ class GetCellRuntimeDataData:
     session_id: str
     cell_id: str
     code: Optional[str] = None
-    errors: Optional[CellErrors] = None
+    errors: Optional[list[MarimoErrorDetail]] = None
     metadata: Optional[CellRuntimeMetadata] = None
     variables: Optional[CellVariables] = None
 
@@ -106,6 +91,28 @@ class GetCellRuntimeDataOutput(SuccessResult):
     )
 
 
+@dataclass
+class CellVisualOutput:
+    """Visual from a cell execution."""
+
+    visual_output: Optional[str] = None
+    visual_mimetype: Optional[str] = None
+
+
+@dataclass
+class GetCellOutputArgs:
+    session_id: SessionId
+    cell_id: CellId_t
+
+
+@dataclass
+class GetCellOutputOutput(SuccessResult):
+    visual_output: CellVisualOutput = field(default_factory=CellVisualOutput)
+    console_outputs: MarimoCellConsoleOutputs = field(
+        default_factory=MarimoCellConsoleOutputs
+    )
+
+
 class GetLightweightCellMap(
     ToolBase[GetLightweightCellMapArgs, GetLightweightCellMapOutput]
 ):
@@ -122,6 +129,16 @@ class GetLightweightCellMap(
         A success result containing lightweight cell previews and navigation info.
     """
 
+    guidelines = ToolGuidelines(
+        when_to_use=[
+            "To get an overview of notebook structure and all cell IDs",
+            "When navigating a notebook before making targeted changes",
+        ],
+        prerequisites=[
+            "You must have a valid session id from an active notebook",
+        ],
+    )
+
     def handle(
         self, args: GetLightweightCellMapArgs
     ) -> GetLightweightCellMapOutput:
@@ -217,6 +234,17 @@ class GetCellRuntimeData(
         A success result containing cell runtime data including code, errors, and variables.
     """
 
+    guidelines = ToolGuidelines(
+        when_to_use=[
+            "When inspecting a specific cell's code, errors, or variables",
+            "After identifying a cell of interest from the cell map",
+        ],
+        prerequisites=[
+            "You must have a valid session id from an active notebook",
+            "You must have a valid cell id from an active notebook",
+        ],
+    )
+
     def handle(self, args: GetCellRuntimeDataArgs) -> GetCellRuntimeDataOutput:
         session_id = args.session_id
         cell_id = args.cell_id
@@ -230,7 +258,7 @@ class GetCellRuntimeData(
         cell_code = cell_data.code
 
         # Get cell errors from session view with actual error details
-        cell_errors = self._get_cell_errors(session, cell_id)
+        cell_errors = context.get_cell_errors(session_id, cell_id)
 
         # Get cell runtime metadata
         cell_metadata = self._get_cell_metadata(session, cell_id)
@@ -270,79 +298,6 @@ class GetCellRuntimeData(
             )
         return cell_data
 
-    def _get_cell_errors(
-        self, session: Session, cell_id: CellId_t
-    ) -> CellErrors:
-        """Get cell errors from session view with actual error details."""
-        from marimo._messaging.cell_output import CellChannel
-
-        # Get cell operation from session view
-        session_view = session.session_view
-        cell_op = session_view.cell_operations.get(cell_id)
-
-        if cell_op is None:
-            # No operations recorded for this cell
-            return CellErrors(has_errors=False, error_details=None)
-
-        # Check for actual error details in the output
-        has_errors = False
-        error_details = []
-        if (
-            cell_op.output
-            and cell_op.output.channel == CellChannel.MARIMO_ERROR
-        ):
-            has_errors = True
-            # Extract actual error objects
-            errors = cell_op.output.data
-            if isinstance(errors, list):
-                for error in errors:
-                    if hasattr(error, "type") and hasattr(error, "describe"):
-                        # Rich Error object
-                        error_detail = ErrorDetail(
-                            type=error.type,
-                            message=error.describe(),
-                            traceback=getattr(error, "traceback", []),
-                        )
-                        error_details.append(error_detail)
-                    elif isinstance(error, dict):
-                        # Dict-based error
-                        dict_error_detail = ErrorDetail(
-                            type=error.get("type", "UnknownError"),
-                            message=error.get("msg", str(error)),
-                            traceback=error.get("traceback", []),
-                        )
-                        error_details.append(dict_error_detail)
-                    else:
-                        # Fallback for other error types
-                        fallback_error_detail = ErrorDetail(
-                            type=type(error).__name__,
-                            message=str(error),
-                            traceback=[],
-                        )
-                        error_details.append(fallback_error_detail)
-
-        # Check console outputs for STDERR (includes print statements to stderr, warnings, etc.)
-        if cell_op.console:
-            console_outputs = (
-                cell_op.console
-                if isinstance(cell_op.console, list)
-                else [cell_op.console]
-            )
-            for console_output in console_outputs:
-                if console_output.channel == CellChannel.STDERR:
-                    has_errors = True
-                    stderr_error_detail = ErrorDetail(
-                        type="STDERR",
-                        message=str(console_output.data),
-                        traceback=[],
-                    )
-                    error_details.append(stderr_error_detail)
-
-        return CellErrors(
-            has_errors=has_errors,
-            error_details=error_details if error_details else None,
-        )
-
     def _get_cell_metadata(
         self, session: Session, cell_id: CellId_t
     ) -> CellRuntimeMetadata:
@@ -381,10 +336,78 @@ class GetCellRuntimeData(
         for var_name in cell_defs:
             if var_name in all_variables:
                 var_value = all_variables[var_name]
-                cell_variables[var_name] = CellVariableValue(
-                    name=var_name,
-                    value=var_value.value,
-                    data_type=var_value.datatype,
-                )
+                cell_variables[var_name] = var_value
 
         return cell_variables
+
+
+class GetCellOutputs(ToolBase[GetCellOutputArgs, GetCellOutputOutput]):
+    """Get cell execution output including visual display and console streams.
+
+    Args:
+        session_id: The session ID of the notebook from get_active_notebooks
+        cell_id: The specific cell ID from get_lightweight_cell_map
+
+    Returns:
+        Visual output (HTML, charts, tables, etc.) with mimetype and console streams (stdout/stderr).
+    """
+
+    guidelines = ToolGuidelines(
+        when_to_use=[
+            "When you need to see what a cell displayed or printed to the user",
+            "To review charts, visualizations, markdown, HTML, or console output from a cell",
+        ],
+        prerequisites=[
+            "You must have a valid session id from an active notebook",
+            "You must have a valid cell id from an active notebook",
+        ],
+    )
+
+    def handle(self, args: GetCellOutputArgs) -> GetCellOutputOutput:
+        context = self.context
+        session = context.get_session(args.session_id)
+        session_view = session.session_view
+        cell_id = args.cell_id
+        cell_op = session_view.cell_operations.get(cell_id)
+
+        if cell_op is None:
+            raise ToolExecutionError(
+                f"Cell {cell_id} not found in session {args.session_id}",
+                code="CELL_NOT_FOUND",
+                is_retryable=False,
+                suggested_fix="Use get_lightweight_cell_map to find valid cell IDs",
+            )
+
+        visual_output, visual_mimetype = self._get_visual_output(cell_op)
+        console_outputs = context.get_cell_console_outputs(cell_op)
+
+        return GetCellOutputOutput(
+            visual_output=CellVisualOutput(
+                visual_output=visual_output,
+                visual_mimetype=visual_mimetype,
+            ),
+            console_outputs=console_outputs,
+            next_steps=[
+                "Review visual_output to see what was displayed to the user",
+                "Check stdout/stderr for print statements and warnings",
+            ],
+        )
+
+    def _get_visual_output(
+        self, cell_op: CellOp
+    ) -> tuple[Optional[str], Optional[str]]:
+        visual_output = None
+        visual_mimetype = None
+        if cell_op.output:
+            data = cell_op.output.data
+            visual_output = self._get_str_output_data(data)
+            visual_mimetype = cell_op.output.mimetype
+        return visual_output, visual_mimetype
+
+    def _get_str_output_data(
+        self, data: str | list[Error] | dict[str, Any]
+    ) -> str:
+        if isinstance(data, str):
+            return data
+        else:
+            return str(data)
