@@ -122,6 +122,7 @@ RunResultStatusType = Literal[
 @dataclasses.dataclass
 class RunResultStatus:
     state: Optional[RunResultStatusType] = None
+    exception: Optional[Exception] = None
 
 
 @dataclasses.dataclass
@@ -173,6 +174,9 @@ class CellImpl:
     # unique id
     cell_id: CellId_t
 
+    # Markdown content of the cell if it exists
+    markdown: Optional[str] = None
+
     # Mutable fields
     # explicit configuration of cell
     config: CellConfig = dataclasses.field(default_factory=CellConfig)
@@ -351,9 +355,12 @@ class CellImpl:
         )
 
     def set_run_result_status(
-        self, run_result_status: RunResultStatusType
+        self,
+        run_result_status: RunResultStatusType,
+        exception: Exception | None = None,
     ) -> None:
         self._run_result_status.state = run_result_status
+        self._run_result_status.exception = exception
 
     def set_stale(
         self, stale: bool, stream: Stream | None = None, broadcast: bool = True
@@ -373,6 +380,10 @@ class CellImpl:
     def output(self) -> Any:
         return self._output.output
 
+    @property
+    def exception(self) -> Exception | None:
+        return self._run_result_status.exception
+
 
 @dataclasses.dataclass
 class Cell:
