@@ -14,8 +14,6 @@ import click
 import marimo._cli.cli_validators as validators
 from marimo import _loggers
 from marimo._ast import codegen
-from marimo._ast.load import get_notebook_status
-from marimo._ast.parse import MarimoFileError
 from marimo._cli.config.commands import config
 from marimo._cli.convert.commands import convert
 from marimo._cli.development.commands import development
@@ -23,17 +21,21 @@ from marimo._cli.envinfo import get_system_info
 from marimo._cli.export.commands import export
 from marimo._cli.file_path import validate_name
 from marimo._cli.parse_args import parse_args
-from marimo._cli.print import bold, green, red
+from marimo._cli.print import red
 from marimo._cli.run_docker import (
     prompt_run_in_docker_container,
 )
 from marimo._cli.upgrade import check_for_updates, print_latest_version
+from marimo._cli.utils import (
+    check_app_correctness,
+    check_app_correctness_or_convert,
+    resolve_token,
+)
 from marimo._config.settings import GLOBAL_SETTINGS
 from marimo._lint import run_check
 from marimo._server.file_router import AppFileRouter
 from marimo._server.model import SessionMode
 from marimo._server.start import start
-from marimo._server.tokens import AuthToken
 from marimo._tutorials import (
     Tutorial,
     create_temp_tutorial_file,
@@ -58,84 +60,6 @@ def helpful_usage_error(self: Any, file: Any = None) -> None:
         click.echo(self.ctx.get_help(), file=file, color=color)
 
 
-def check_app_correctness(filename: str, noninteractive: bool = True) -> None:
-    try:
-        status = get_notebook_status(filename).status
-    except (SyntaxError, MarimoFileError):
-        # Exit early if we can
-        if not noninteractive:
-            raise
-
-        # This prints a more readable error message, without internal details
-        # e.g.
-        # Error:   File "/my/bad/file.py", line 17
-        #     x.
-        #     ^
-        # SyntaxError: invalid syntax
-        from marimo._lint import collect_messages
-
-        _, message = collect_messages(filename)
-        raise click.ClickException(message.strip()) from None
-
-    if status == "invalid" and filename.endswith(".py"):
-        # fail for python scripts, almost certainly do not want to override contents
-        import os
-
-        stem = os.path.splitext(os.path.basename(filename))[0]
-        raise click.ClickException(
-            f"Python script not recognized as a marimo notebook.\n\n"
-            f"  {green('Tip:')} Try converting with"
-            "\n\n"
-            f"    marimo convert {filename} -o {stem}_nb.py\n\n"
-            f"  then open with marimo edit {stem}_nb.py"
-        ) from None
-
-    # Only show the tip if we're in an interactive terminal
-    interactive = sys.stdin.isatty() and not noninteractive
-    if status == "invalid" and interactive:
-        click.echo(
-            green("tip")
-            + ": Use `"
-            + bold("marimo convert")
-            + "` to convert existing scripts.",
-            err=True,
-        )
-        click.confirm(
-            (
-                "The file is not detected as a marimo notebook, opening it may "
-                "overwrite its contents.\nDo you want to open it anyway?"
-            ),
-            default=False,
-            abort=True,
-        )
-
-    if status == "has_errors":
-        # Provide a warning, but allow the user to open the notebook
-        _loggers.marimo_logger().warning(
-            "This notebook has errors, saving may lose data. Continuing anyway."
-        )
-
-
-def check_app_correctness_or_convert(filename: str) -> None:
-    from marimo._convert.converters import MarimoConvert
-
-    file = Path(filename)
-    code = file.read_text(encoding="utf-8")
-    try:
-        return check_app_correctness(filename, noninteractive=True)
-    except (click.ClickException, MarimoFileError):
-        # A click exception is raised if a python script could not be converted
-        code = MarimoConvert.from_non_marimo_python_script(
-            source=code, aggressive=True
-        ).to_py()
-    except SyntaxError:
-        # The file could not even be read as python
-        code = MarimoConvert.from_plain_text(source=code).to_py()
-        file.write_text(code, encoding="utf-8")
-
-    file.write_text(code, encoding="utf-8")
-
-
 click.exceptions.UsageError.show = helpful_usage_error  # type: ignore
 
 
@@ -159,18 +83,6 @@ def _key_value_bullets(items: list[tuple[str, str]]) -> str:
     return "\n".join(lines)
 
 
-def _resolve_token(
-    token: bool, token_password: Optional[str]
-) -> Optional[AuthToken]:
-    if token_password:
-        return AuthToken(token_password)
-    elif token is False:
-        # Empty means no auth
-        return AuthToken("")
-    # None means use the default (generated) token
-    return None
-
-
 main_help_msg = "\n".join(
     [
         "\b",
@@ -377,6 +289,13 @@ edit_help_msg = "\n".join(
     type=str,
     help=token_password_message,
 )
+@click.option(
+    "--token-password-file",
+    default=None,
+    show_default=True,
+    type=str,
+    help="Path to file containing token password, or '-' for stdin. Mutually exclusive with --token-password.",
+)
 @click.option(
     "--base-url",
     default="",
@@ -496,6 +415,7 @@ def edit(
     headless: bool,
     token: bool,
     token_password: Optional[str],
+    token_password_file: Optional[str],
     base_url: str,
     allow_origins: Optional[tuple[str, ...]],
     skip_update_check: bool,
@@ -513,8 +433,15 @@ def edit(
     name: Optional[str],
     args: tuple[str, ...],
 ) -> None:
+    from marimo._cli.sandbox import run_in_sandbox, should_run_in_sandbox
+
+    pass_on_stdin = token_password_file == "-"
     # We support unix-style piping, e.g. cat notebook.py | marimo edit
-    if name is None and (stdin_contents := _get_stdin_contents()) is not None:
+    if (
+        not pass_on_stdin
+        and name is None
+        and (stdin_contents := _get_stdin_contents()) is not None
+    ):
         temp_dir = tempfile.TemporaryDirectory()
         path = create_temp_notebook_file(
             "notebook.py", "py", stdin_contents, temp_dir
@@ -529,50 +456,15 @@ def edit(
 
         run_in_docker(
             name,
+            "edit",
             port=port,
             debug=GLOBAL_SETTINGS.DEVELOPMENT_MODE,
         )
         return
 
-    # Dangerous sandbox can be forced on by setting an environment variable;
-    # this allows our VS Code extension to force sandbox regardless of the
-    # marimo version.
-    if sandbox and os.getenv("MARIMO_DANGEROUS_SANDBOX"):
-        dangerous_sandbox = True
-
-    if dangerous_sandbox and (name is None or os.path.isdir(name)):
-        sandbox = True
-        click.echo(
-            click.style(
-                "Warning: Using sandbox with multi-notebook edit servers is dangerous.\n",
-                fg="yellow",
-            )
-            + "Notebook dependencies may not be respected, may not be written, and may be overwritten.\n"
-            + "Learn more: https://github.com/marimo-team/marimo/issues/5219l.\n",
-            err=True,
-        )
-
-    if sandbox is None:
-        # When the sandbox flag is omitted we infer whether to
-        # to start in sandbox mode by examining the notebook file and
-        # prompting the user.
-        from marimo._cli.sandbox import maybe_prompt_run_in_sandbox
-
-        sandbox = maybe_prompt_run_in_sandbox(name)
-    elif (
-        sandbox
-        and not dangerous_sandbox
-        and (name is None or os.path.isdir(name))
+    if should_run_in_sandbox(
+        sandbox=sandbox, dangerous_sandbox=dangerous_sandbox, name=name
     ):
-        raise click.UsageError(
-            """marimo's package sandbox requires a notebook name:
-
-    * marimo edit --sandbox my_notebook.py
-
-  Multi-notebook sandboxed servers (marimo edit --sandbox) are not supported.
-  Follow this issue at: https://github.com/marimo-team/marimo/issues/2598."""
-        )
-    elif sandbox:
         from marimo._cli.sandbox import run_in_sandbox
 
         # TODO: consider adding recommended as well
@@ -631,7 +523,11 @@ def edit(
         skew_protection=skew_protection,
         cli_args=parse_args(args),
         argv=list(args),
-        auth_token=_resolve_token(token, token_password),
+        auth_token=resolve_token(
+            token,
+            token_password=token_password,
+            token_password_file=token_password_file,
+        ),
         base_url=base_url,
         allow_origins=allow_origins,
         redirect_console_to_browser=True,
@@ -715,6 +611,13 @@ new_help_msg = "\n".join(
     type=str,
     help=token_password_message,
 )
+@click.option(
+    "--token-password-file",
+    default=None,
+    show_default=True,
+    type=str,
+    help="Path to file containing token password, or '-' for stdin. Mutually exclusive with --token-password.",
+)
 @click.option(
     "--base-url",
     default="",
@@ -755,6 +658,7 @@ def new(
     headless: bool,
     token: bool,
     token_password: Optional[str],
+    token_password_file: Optional[str],
     base_url: str,
     sandbox: Optional[bool],
     skew_protection: bool,
@@ -833,7 +737,11 @@ def new(
         skew_protection=skew_protection,
         cli_args={},
         argv=[],
-        auth_token=_resolve_token(token, token_password),
+        auth_token=resolve_token(
+            token,
+            token_password=token_password,
+            token_password_file=token_password_file,
+        ),
         base_url=base_url,
         redirect_console_to_browser=True,
         ttl_seconds=None,
@@ -894,6 +802,13 @@ Example:
     type=str,
     help=token_password_message,
 )
+@click.option(
+    "--token-password-file",
+    default=None,
+    show_default=True,
+    type=str,
+    help="Path to file containing token password, or '-' for stdin. Mutually exclusive with --token-password.",
+)
 @click.option(
     "--include-code",
     is_flag=True,
@@ -994,6 +909,7 @@ def run(
     headless: bool,
     token: bool,
     token_password: Optional[str],
+    token_password_file: Optional[str],
     include_code: bool,
     session_ttl: int,
     watch: bool,
@@ -1008,6 +924,8 @@ def run(
     name: str,
     args: tuple[str, ...],
 ) -> None:
+    from marimo._cli.sandbox import run_in_sandbox, should_run_in_sandbox
+
     # If file is a url, we prompt to run in docker
     # We only do this for remote files,
     # but later we can make this a CLI flag
@@ -1016,20 +934,15 @@ def run(
 
         run_in_docker(
             name,
+            "run",
             port=port,
             debug=GLOBAL_SETTINGS.DEVELOPMENT_MODE,
         )
         return
 
-    # Set default, if not provided
-    if sandbox is None:
-        from marimo._cli.sandbox import maybe_prompt_run_in_sandbox
-
-        sandbox = maybe_prompt_run_in_sandbox(name)
-
-    if sandbox:
-        from marimo._cli.sandbox import run_in_sandbox
-
+    if should_run_in_sandbox(
+        sandbox=sandbox, dangerous_sandbox=None, name=name
+    ):
         run_in_sandbox(sys.argv[1:], name=name)
         return
 
@@ -1070,7 +983,11 @@ def run(
         allow_origins=allow_origins,
         cli_args=parse_args(args),
         argv=list(args),
-        auth_token=_resolve_token(token, token_password),
+        auth_token=resolve_token(
+            token,
+            token_password=token_password,
+            token_password_file=token_password_file,
+        ),
         redirect_console_to_browser=redirect_console_to_browser,
         server_startup_command=server_startup_command,
         asset_url=asset_url,
@@ -1148,6 +1065,13 @@ Recommended sequence:
     type=str,
     help=token_password_message,
 )
+@click.option(
+    "--token-password-file",
+    default=None,
+    show_default=True,
+    type=str,
+    help="Path to file containing token password, or '-' for stdin. Mutually exclusive with --token-password.",
+)
 @click.option(
     "--skew-protection/--no-skew-protection",
     is_flag=True,
@@ -1168,6 +1092,7 @@ def tutorial(
     headless: bool,
     token: bool,
     token_password: Optional[str],
+    token_password_file: Optional[str],
     skew_protection: bool,
     name: Tutorial,
 ) -> None:
@@ -1188,7 +1113,11 @@ def tutorial(
         skew_protection=skew_protection,
         cli_args={},
         argv=[],
-        auth_token=_resolve_token(token, token_password),
+        auth_token=resolve_token(
+            token,
+            token_password=token_password,
+            token_password_file=token_password_file,
+        ),
         redirect_console_to_browser=False,
         ttl_seconds=None,
     )
@@ -1281,33 +1210,64 @@ def shell_completion() -> None:
     type=bool,
     help="Enable fixes that may change code behavior (e.g., removing empty cells).",
 )
+@click.option(
+    "--ignore-scripts",
+    is_flag=True,
+    default=False,
+    show_default=True,
+    type=bool,
+    help="Ignore files that are not recognizable as marimo notebooks.",
+)
+@click.option(
+    "--format",
+    "formatter",
+    default="full",
+    show_default=True,
+    type=click.Choice(["full", "json"], case_sensitive=False),
+    help="Output format for diagnostics.",
+)
 @click.argument("files", nargs=-1, type=click.UNPROCESSED)
 def check(
     fix: bool,
     strict: bool,
     verbose: bool,
     unsafe_fixes: bool,
+    ignore_scripts: bool,
+    formatter: str,
     files: tuple[str, ...],
 ) -> None:
     if not files:
         # If no files are provided, we lint the current directory
         files = ("**/*.py", "**/*.md", "**/*.qmd")
 
-    # Pass click.echo directly as pipe for streaming output, or None
-    pipe = click.echo if verbose else None
-    linter = run_check(files, pipe=pipe, fix=fix, unsafe_fixes=unsafe_fixes)
+    # Pass click.echo directly as pipe for streaming output, or None for JSON
+    pipe = click.echo if verbose and formatter != "json" else None
+    linter = run_check(
+        files,
+        pipe=pipe,
+        fix=fix,
+        unsafe_fixes=unsafe_fixes,
+        ignore_scripts=ignore_scripts,
+        formatter=formatter,
+    )
 
-    # Get counts from linter (fix happens automatically during streaming)
-    fixed = linter.fixed_count
-    total_issues = linter.issues_count
+    if formatter == "json":
+        # JSON output - let linter handle the collection and formatting
+        result = linter.get_json_result()
+        # Always output to stdout for JSON, regardless of errors
+        click.echo(json.dumps(result), err=False)
+    else:
+        # Get counts from linter (fix happens automatically during streaming)
+        fixed = linter.fixed_count
+        total_issues = linter.issues_count
 
-    # Final summary
-    if fixed > 0:
-        click.echo(f"Updated {fixed} file{'s' if fixed > 1 else ''}.")
-    if total_issues > 0:
-        click.echo(
-            f"Found {total_issues} issue{'s' if total_issues > 1 else ''}."
-        )
+        # Final summary
+        if fixed > 0:
+            click.echo(f"Updated {fixed} file{'s' if fixed > 1 else ''}.")
+        if total_issues > 0:
+            click.echo(
+                f"Found {total_issues} issue{'s' if total_issues > 1 else ''}."
+            )
 
     if linter.errored or (strict and (fixed > 0 or total_issues > 0)):
         sys.exit(1)
