@@ -49,6 +49,7 @@ def _generate_server_api_schema() -> dict[str, Any]:
     from marimo._messaging.mimetypes import KnownMimeType
     from marimo._plugins.core.web_component import JSONType
     from marimo._runtime.packages.package_manager import PackageDescription
+    from marimo._server.ai.tools.types import ToolDefinition
     from marimo._server.api.router import build_routes
     from marimo._utils.dataclass_to_openapi import (
         PythonTypeToOpenAPI,
@@ -126,6 +127,8 @@ def _generate_server_api_schema() -> dict[str, Any]:
         ops.SQLTableListPreview,
         ops.DataSourceConnections,
         ops.SecretKeysResult,
+        ops.CacheCleared,
+        ops.CacheInfoFetched,
         ops.QueryParamsSet,
         ops.QueryParamsAppend,
         ops.QueryParamsDelete,
@@ -136,6 +139,7 @@ def _generate_server_api_schema() -> dict[str, Any]:
         ops.MessageOperation,
         # ai
         ChatMessage,
+        ToolDefinition,
     ]
 
     # dataclass components used in requests/responses
@@ -207,6 +211,8 @@ def _generate_server_api_schema() -> dict[str, Any]:
         models.UpdateComponentValuesRequest,
         models.InvokeAiToolRequest,
         models.InvokeAiToolResponse,
+        models.MCPStatusResponse,
+        models.MCPRefreshResponse,
         requests.CodeCompletionRequest,
         requests.DeleteCellRequest,
         requests.HTTPRequest,
@@ -217,11 +223,14 @@ def _generate_server_api_schema() -> dict[str, Any]:
         requests.FunctionCallRequest,
         requests.InstallMissingPackagesRequest,
         requests.ListSecretKeysRequest,
+        requests.ClearCacheRequest,
+        requests.GetCacheInfoRequest,
         requests.PdbRequest,
         requests.PreviewDatasetColumnRequest,
         requests.PreviewSQLTableListRequest,
         requests.PreviewDataSourceConnectionRequest,
         requests.PreviewSQLTableRequest,
+        requests.ValidateSQLRequest,
         requests.RenameRequest,
         requests.SetCellConfigRequest,
         requests.ModelMessage,
@@ -290,7 +299,7 @@ def _generate_server_api_schema() -> dict[str, Any]:
         component_schemas[name] = converter.convert(cls, processed_classes)
         processed_classes[cls] = name
 
-    schemas_geneartor = SchemaGenerator(
+    schemas_generator = SchemaGenerator(
         {
             "openapi": "3.1.0",
             "info": {"title": "marimo API", "version": __version__},
@@ -302,7 +311,7 @@ def _generate_server_api_schema() -> dict[str, Any]:
         }
     )
 
-    schemas = schemas_geneartor.get_schema(routes=build_routes())
+    schemas = schemas_generator.get_schema(routes=build_routes())
 
     # Find/replace #/$defs with #/components/schemas
     import json
