@@ -53,7 +53,9 @@ from marimo._runtime.context.types import ContextNotInitializedError
 from marimo._runtime.context.utils import get_mode
 from marimo._runtime.layout.layout import LayoutConfig
 from marimo._secrets.models import SecretKeysWithProvider
+from marimo._sql.parse import SqlCatalogCheckResult, SqlParseResult
 from marimo._types.ids import CellId_t, RequestId, WidgetModelId
+from marimo._utils.msgspec_basestruct import BaseStruct
 from marimo._utils.platform import is_pyodide, is_windows
 
 LOGGER = loggers.marimo_logger()
@@ -381,7 +383,7 @@ class SendUIElementMessage(Op, tag="send-ui-element-message"):
     ui_element: Optional[str]
     model_id: Optional[WidgetModelId]
     message: dict[str, Any]
-    buffers: Optional[list[str]] = None
+    buffers: Optional[list[bytes]] = None
 
 
 class Interrupted(Op, tag="interrupted"):
@@ -501,7 +503,7 @@ class VariableDeclaration(msgspec.Struct):
     used_by: list[CellId_t]
 
 
-class VariableValue(msgspec.Struct):
+class VariableValue(BaseStruct):
     name: str
     value: Optional[str]
     datatype: Optional[str]
@@ -612,7 +614,7 @@ class ColumnPreview(msgspec.Struct):
 
 
 # We shouldn't need to make table_name and column_name have default values.
-# We can use kw_only=True once we drop support for Python 3.9.
+# We can use kw_only=True once we drop support for Python 3.9 (25-11-01).
 class DataColumnPreview(Op, ColumnPreview, tag="data-column-preview"):
     """Preview of a column in a dataset."""
 
@@ -626,6 +628,14 @@ class DataSourceConnections(Op, tag="data-source-connections"):
     connections: list[DataSourceConnection]
 
 
+class ValidateSQLResult(Op, tag="validate-sql-result"):
+    name: ClassVar[str] = "validate-sql-result"
+    request_id: RequestId
+    parse_result: Optional[SqlParseResult] = None
+    validate_result: Optional[SqlCatalogCheckResult] = None
+    error: Optional[str] = None
+
+
 class QueryParamsSet(Op, tag="query-params-set"):
     """Set query parameters."""
 
@@ -675,6 +685,24 @@ class SecretKeysResult(Op, tag="secret-keys-result"):
     secrets: list[SecretKeysWithProvider]
 
 
+class CacheCleared(Op, tag="cache-cleared"):
+    """Result of clearing cache."""
+
+    name: ClassVar[str] = "cache-cleared"
+    bytes_freed: int
+
+
+class CacheInfoFetched(Op, tag="cache-info-fetched"):
+    """Cache statistics information."""
+
+    name: ClassVar[str] = "cache-info-fetched"
+    hits: int
+    misses: int
+    time: float
+    disk_to_free: int
+    disk_total: int
+
+
 class UpdateCellIdsRequest(Op, tag="update-cell-ids"):
     """
     Update the cell ID ordering of the cells in the notebook.
@@ -721,8 +749,12 @@ MessageOperation = Union[
     SQLTablePreview,
     SQLTableListPreview,
     DataSourceConnections,
+    ValidateSQLResult,
     # Secrets
     SecretKeysResult,
+    # Cache
+    CacheCleared,
+    CacheInfoFetched,
     # Kiosk specific
     FocusCell,
     UpdateCellCodes,
