@@ -1,12 +1,11 @@
 # Copyright 2024 Marimo. All rights reserved.
 from __future__ import annotations
 
-import os
 import re
 from enum import Enum
 from typing import Any
 
-import narwhals.stable.v1 as nw
+import narwhals.stable.v2 as nw
 
 from marimo import _loggers
 from marimo._messaging.mimetypes import KnownMimeType
@@ -14,6 +13,7 @@ from marimo._output.formatters.formatter_factory import (
     FormatterFactory,
     Unregister,
 )
+from marimo._output.hypertext import is_no_js
 from marimo._plugins.stateless.json_output import json_output
 from marimo._plugins.stateless.mermaid import mermaid
 from marimo._plugins.stateless.plain_text import plain_text
@@ -30,7 +30,7 @@ def include_opinionated() -> bool:
         runtime_context_installed,
     )
 
-    if os.getenv("MARIMO_NO_JS", "false").lower() == "true":
+    if is_no_js():
         return False
 
     if runtime_context_installed():
