@@ -6,7 +6,7 @@ import io
 from functools import cached_property
 from typing import Any, Optional, Union
 
-import narwhals.stable.v1 as nw
+import narwhals.stable.v2 as nw
 
 from marimo import _loggers
 from marimo._data.models import (
@@ -38,7 +38,7 @@ class PolarsTableManagerFactory(TableManagerFactory):
         import polars as pl
 
         class PolarsTableManager(
-            NarwhalsTableManager[Union[pl.DataFrame, pl.LazyFrame]]
+            NarwhalsTableManager[pl.DataFrame, pl.LazyFrame]
         ):
             type = "polars"
 
@@ -154,6 +154,15 @@ class PolarsTableManagerFactory(TableManagerFactory):
                                 result, column
                             )
                             converted_columns.append(column.name)
+                        # https://github.com/marimo-team/marimo/issues/5562
+                        elif isinstance(dtype, pl.List) and isinstance(
+                            dtype.inner, (pl.Enum, pl.Categorical)
+                        ):
+                            # Convert each element in the list to a string
+                            result = result.with_columns(
+                                pl.col(column.name).cast(pl.List(pl.String))
+                            )
+                            converted_columns.append(column.name)
 
                     if converted_columns:
                         LOGGER.info(
