@@ -1,11 +1,14 @@
 # Copyright 2024 Marimo. All rights reserved.
 from __future__ import annotations
 
+import ast
 import html
+import sys
 import threading
 import time
+from collections.abc import Collection, Mapping
 from functools import lru_cache
-from typing import TYPE_CHECKING, Any, cast
+from typing import TYPE_CHECKING, Any, Protocol, cast
 
 import jedi  # type: ignore # noqa: F401
 import jedi.api  # type: ignore # noqa: F401
@@ -25,6 +28,7 @@ from marimo._utils.rst_to_html import convert_rst_to_html
 
 if TYPE_CHECKING:
     import threading
+    from types import ModuleType
 
 LOGGER = loggers.marimo_logger()
 
@@ -344,6 +348,190 @@ def _get_completions_with_interpreter(
     return script, completions
 
 
+# TODO move this to a global utility module
+def _isinstance_external(obj: Any, *, class_ref: str) -> bool:
+    """Check if an object is an instance of a class reference defined by a string.
+    Allows to define and do `isinstance()` checks without importing 3rd party libraries
+
+    Example:
+        ```python
+        df = pd.DataFrame(...)
+        _isinstance_external(df, class_ref="pandas.DataFrame")
+        ```
+    """
+    import_parts = class_ref.split(".")
+    module_name = import_parts[0]
+
+    if module_name not in sys.modules:
+        return False
+
+    module: ModuleType = sys.modules[module_name]
+    target_class: Any = module
+    for part in import_parts:
+        target_class = getattr(target_class, part)
+
+    return isinstance(obj, target_class)
+
+
+class HasKeysMethod(Protocol):
+    def keys(self) -> Collection[Any]: ...
+
+
+class HasColumnsProperty(Protocol):
+    @property
+    def columns(self) -> Collection[Any]: ...
+
+
+def _key_options_from_ipython_method(obj: Any) -> list[str]:
+    # convention for `_ipython_key_completions_` is to return a list of strings
+    return [str(key) for key in obj._ipython_key_completions_()]
+
+
+def _key_options_via_keys_method(obj: HasKeysMethod) -> list[str]:
+    return [str(key) for key in obj.keys()]
+
+
+# TODO refactor to customize the `CompletionOption.info` with `"columns"`
+def _key_options_via_columns_method(obj: HasColumnsProperty) -> list[str]:
+    return [str(col) for col in obj.columns]
+
+
+def _key_options_dispatcher(obj: Any) -> list[str]:
+    """Tries to get key completion suggestions from `obj`
+    based on its methods and type.
+
+    Returns a list of strings to later create marimo `CompletionOption`
+    """
+    if getattr(obj, "_ipython_key_completions_", False):
+        return _key_options_from_ipython_method(obj)
+    elif isinstance(obj, Mapping):
+        return _key_options_via_keys_method(obj)
+    elif _isinstance_external(obj, class_ref="pandas.DataFrame"):
+        return _key_options_via_columns_method(obj)
+    elif _isinstance_external(obj, class_ref="polars.DataFrame"):
+        return _key_options_via_columns_method(obj)
+
+    LOGGER.debug(
+        f"No matching handlers found to retrieve keys from type `{type(obj)}`"
+    )
+    return []
+
+
+# NOTE need to be careful because `__getitem__` can trigger arbitrary code
+def _resolve_chained_key_path(obj_name: str, document: str) -> list[list[str]]:
+    """Resolve the object chained key access from source code
+
+    Example:
+        ```python
+        key_path = _resolve_chained_key_path("obj", 'obj["foo"]["bar"]["')
+        key_path == [["foo"], ["bar"]]
+        ```
+    """
+    import parso  # jedi dependency
+    from parso.utils import split_lines
+
+    # because marimo `CompletionRequest` sends source code up to the cursor position
+    # we know only the last line matters for autcompletion.
+    line_containing_trigger = split_lines(document)[-1]
+    ast_ = parso.parse(line_containing_trigger)  # type: ignore[no-untyped-call]
+    # we expect to always hit an error node because `dictionary["` is invalid Python syntax
+    root_node = next(
+        node for node in ast_.children if node.type == "error_node"
+    )
+
+    key_path = []
+    seen_object_node = False
+    # TODO robust handling of different node types
+    for node in root_node.children:
+        if obj_name in node.get_code():
+            seen_object_node = True
+            continue
+
+        # iterate until we find the node associated with `obj_name`
+        if seen_object_node is False:
+            continue
+
+        # if nodes directly after `obj_name` node are not key accessor `[""]`, exit
+        # we expect to never hit this condition
+        if not node.type == "trailer":
+            break
+
+        key_path.append(ast.literal_eval(node.get_code()))
+
+    return key_path
+
+
+def _get_key_options(
+    script: jedi.Script,
+    glbls: dict[str, Any],
+    document: str,
+) -> list[CompletionOption]:
+    """Get completion values for trigger `["` or `['`. Values are meant to be
+    passed to `.__getitem__()`
+    """
+    names = script.get_names(
+        references=True, definitions=False, all_scopes=False
+    )
+    if not names:
+        LOGGER.debug(
+            f"Retrieved no `names` for completion request: `{script._code}`"
+        )
+        return []
+
+    obj_name = names[-1].name
+    root_obj = glbls.get(obj_name)
+    if root_obj is None:
+        LOGGER.debug(f"Failed to find `{obj_name=}` in `glbls`")
+        return []
+
+    obj = root_obj
+    for key in _resolve_chained_key_path(obj_name, document):
+        try:
+            obj = obj.__getitem__(*key)
+        except Exception:
+            LOGGER.debug(
+                f"Failed to retrieve keys `{key}` based on `{document=}`"
+            )
+            # exit early if `__getitem__` fails and return no completion
+            # e.g., no completion is possible if key `"foo"` is invalid in `dictionary["foo"]["`
+            return []
+
+    key_options = _key_options_dispatcher(obj)
+    # TODO currently unreliable for non-string keys, even if stringified
+    # seems to be related to serialization issue if include `"(True, False)` (no closing quote)
+    return [
+        CompletionOption(
+            name=key_option, type="property", completion_info="key"
+        )
+        for key_option in key_options
+    ]
+
+
+def _maybe_get_key_options(
+    document: str,
+    script: jedi.Script,
+    glbls: dict[str, Any],
+    glbls_lock: threading.RLock,
+) -> list[CompletionOption]:
+    """Call key completions methods if the request contains the trigger"""
+    triggers = ('["', "['")  # explicitly handle both quotation characters
+    if document[-2:] not in triggers:
+        return []
+
+    # TODO ideally, we don't acquire the lock twice with `_get_completions_with_interpreter()` and `_maybe_get_key_options`
+    # however, the two functions are applied on different triggers
+    # also, this other function returns Jedi `Completion` while this returns native `CompletionOption`
+    locked = False
+    locked = glbls_lock.acquire(blocking=False)
+    if locked:
+        try:
+            return _get_key_options(script, glbls, document)
+        finally:
+            glbls_lock.release()
+    else:
+        return []
+
+
 def _get_completions(
     codes: list[str],
     document: str,
@@ -359,6 +547,9 @@ def _get_completions(
     return script, completions
 
 
+# NOTE you will hit front display bug if the result set doesn't
+# share the same `prefix_length`. This could happen if completion values
+# are generated via different means
 def complete(
     request: CodeCompletionRequest,
     graph: dataflow.DirectedGraph,
@@ -411,6 +602,24 @@ def complete(
         prefix_length: int = (
             completions[0].get_completion_prefix_length() if completions else 0
         )
+        prefix = request.document[-prefix_length:]
+
+        key_options = _maybe_get_key_options(
+            request.document,
+            script,
+            glbls=glbls,
+            glbls_lock=glbls_lock,
+        )
+        # if `key_options` are found, return early
+        # we can return because key and attribute completions have non-overlapping triggers
+        if key_options:
+            _write_completion_result(
+                stream=stream,
+                completion_id=request.id,
+                prefix_length=0,  # values is static because of our implementation
+                options=key_options,
+            )
+            return
 
         if (
             prefix_length == 0
@@ -448,7 +657,6 @@ def complete(
             _write_no_completions(stream, request.id)
             return
 
-        prefix = request.document[-prefix_length:]
         if timeout is None and isinstance(script, jedi.Interpreter):
             # We're holding the globals lock; set a short timeout so we don't
             # block the kernel
