@@ -39,7 +39,9 @@ class PypiPackageManager(CanonicalizingPackageManager):
     ) -> list[PackageDescription]:
         if not self.is_manager_installed():
             return []
-        proc = subprocess.run(cmd, capture_output=True, text=True)
+        proc = subprocess.run(
+            cmd, capture_output=True, text=True, encoding="utf-8"
+        )
         if proc.returncode != 0:
             return []
         try:
@@ -56,19 +58,15 @@ class PipPackageManager(PypiPackageManager):
     name = "pip"
     docs_url = "https://pip.pypa.io/"
 
-    async def _install(
-        self,
-        package: str,
-        *,
-        upgrade: bool,
-        log_callback: Optional[LogCallback] = None,
-    ) -> bool:
-        LOGGER.info(f"Installing {package} with pip")
-        cmd = ["pip", "--python", PY_EXE, "install"]
-        if upgrade:
-            cmd.append("--upgrade")
-        cmd.extend(split_packages(package))
-        return self.run(cmd, log_callback=log_callback)
+    def install_command(self, package: str, *, upgrade: bool) -> list[str]:
+        return [
+            "pip",
+            "--python",
+            PY_EXE,
+            "install",
+            *(["--upgrade"] if upgrade else []),
+            *split_packages(package),
+        ]
 
     async def uninstall(self, package: str) -> bool:
         LOGGER.info(f"Uninstalling {package} with pip")
@@ -165,20 +163,11 @@ class UvPackageManager(PypiPackageManager):
     def is_manager_installed(self) -> bool:
         return self._uv_bin != "uv" or super().is_manager_installed()
 
-    async def _install(
-        self,
-        package: str,
-        *,
-        upgrade: bool,
-        log_callback: Optional[LogCallback] = None,
-    ) -> bool:
+    def install_command(self, package: str, *, upgrade: bool) -> list[str]:
         install_cmd: list[str]
         if self.is_in_uv_project:
-            LOGGER.info(f"Installing in {package} with 'uv add'")
             install_cmd = [self._uv_bin, "add"]
         else:
-            LOGGER.info(f"Installing in {package} with 'uv pip install'")
-
             install_cmd = [self._uv_bin, "pip", "install"]
 
             # Allow for explicit site directory location if needed
@@ -189,10 +178,28 @@ class UvPackageManager(PypiPackageManager):
         if upgrade:
             install_cmd.append("--upgrade")
 
-        return self.run(
+        return install_cmd + [
             # trade installation time for faster start time
-            install_cmd
-            + ["--compile", *split_packages(package), "-p", PY_EXE],
+            "--compile",
+            *split_packages(package),
+            "-p",
+            PY_EXE,
+        ]
+
+    async def _install(
+        self,
+        package: str,
+        *,
+        upgrade: bool,
+        log_callback: Optional[LogCallback] = None,
+    ) -> bool:
+        """Installation logic."""
+        LOGGER.info(
+            f"Installing in {package} with 'uv {'add' if self.is_in_uv_project else 'pip install'}'"
+        )
+        return await super()._install(
+            package,
+            upgrade=upgrade,
             log_callback=log_callback,
         )
 
@@ -235,6 +242,26 @@ class UvPackageManager(PypiPackageManager):
 
         version_map = self._get_version_map()
 
+        def _is_direct_reference(package: str) -> bool:
+            """Check if a package is a direct reference (git, URL, or local path).
+
+            Direct references should bypass the _is_installed check because:
+            - Git URLs (git+https://...) won't appear in version_map with that prefix
+            - Direct URL references (package @ https://...) use @ syntax
+            - Local paths (package @ file://...) use @ syntax
+            - These should be passed directly to uv which handles them correctly
+            """
+            # Git URLs: git+https://, git+ssh://, git://
+            if package.startswith("git+") or package.startswith("git://"):
+                return True
+            # Direct references with @ (PEP 440 direct references)
+            if " @ " in package:
+                return True
+            # URLs (https://, http://, file://)
+            if "://" in package:
+                return True
+            return False
+
         def _is_installed(package: str) -> bool:
             without_brackets = package.split("[")[0]
             return without_brackets.lower() in version_map
@@ -249,11 +276,13 @@ class UvPackageManager(PypiPackageManager):
                 return f"{package}=={version}"
             return package
 
-        # Filter to packages that are found in "uv pip list"
+        # Filter to packages that are found in "uv pip list" OR are direct references
+        # Direct references (git URLs, direct URLs, local paths) bypass the installed check
+        # because they won't appear in the version map with their full reference syntax
         packages_to_add = [
-            _maybe_add_version(im)
+            _maybe_add_version(im) if not _is_direct_reference(im) else im
             for im in packages_to_add
-            if _is_installed(im)
+            if _is_direct_reference(im) or _is_installed(im)
         ]
 
         if filepath.endswith(".md") or filepath.endswith(".qmd"):
@@ -454,6 +483,7 @@ class UvPackageManager(PypiPackageManager):
                 tree_cmd,
                 capture_output=True,
                 text=True,
+                encoding="utf-8",
                 check=True,
             )
             tree = parse_uv_tree(result.stdout)
@@ -474,21 +504,12 @@ class RyePackageManager(PypiPackageManager):
     name = "rye"
     docs_url = "https://rye.astral.sh/"
 
-    async def _install(
-        self,
-        package: str,
-        *,
-        upgrade: bool,
-        log_callback: Optional[LogCallback] = None,
-    ) -> bool:
-        if upgrade:
-            return self.run(
-                ["rye", "sync", "--update", *split_packages(package)],
-                log_callback=log_callback,
-            )
-        return self.run(
-            ["rye", "add", *split_packages(package)], log_callback=log_callback
-        )
+    def install_command(self, package: str, *, upgrade: bool) -> list[str]:
+        return [
+            "rye",
+            *(["sync", "--update"] if upgrade else ["add"]),
+            *split_packages(package),
+        ]
 
     async def uninstall(self, package: str) -> bool:
         return self.run(
@@ -504,28 +525,13 @@ class PoetryPackageManager(PypiPackageManager):
     name = "poetry"
     docs_url = "https://python-poetry.org/docs/"
 
-    async def _install(
-        self,
-        package: str,
-        *,
-        upgrade: bool,
-        log_callback: Optional[LogCallback] = None,
-    ) -> bool:
-        if upgrade:
-            return self.run(
-                [
-                    "poetry",
-                    "update",
-                    "--no-interaction",
-                    *split_packages(package),
-                ],
-                log_callback=log_callback,
-            )
-
-        return self.run(
-            ["poetry", "add", "--no-interaction", *split_packages(package)],
-            log_callback=log_callback,
-        )
+    def install_command(self, package: str, *, upgrade: bool) -> list[str]:
+        return [
+            "poetry",
+            "update" if upgrade else "add",
+            "--no-interaction",
+            *split_packages(package),
+        ]
 
     async def uninstall(self, package: str) -> bool:
         return self.run(
@@ -538,7 +544,9 @@ class PoetryPackageManager(PypiPackageManager):
     ) -> list[PackageDescription]:
         if not self.is_manager_installed():
             return []
-        proc = subprocess.run(cmd, capture_output=True, text=True)
+        proc = subprocess.run(
+            cmd, capture_output=True, text=True, encoding="utf-8"
+        )
         if proc.returncode != 0:
             return []
 
