@@ -22,6 +22,7 @@ from marimo._loggers import marimo_logger
 from marimo._messaging.errors import (
     Error,
     MarimoExceptionRaisedError,
+    MarimoSQLError,
     MarimoStrictExecutionError,
     UnknownError,
 )
@@ -38,6 +39,10 @@ from marimo._runtime.executor import (
     get_executor,
 )
 from marimo._runtime.marimo_pdb import MarimoPdb
+from marimo._sql.error_utils import (
+    create_sql_error_from_exception,
+    is_sql_parse_error,
+)
 from marimo._types.ids import CellId_t
 
 LOGGER = marimo_logger()
@@ -60,7 +65,7 @@ def cell_filename(cell_id: CellId_t) -> str:
     return f"<cell-{cell_id}>"
 
 
-ErrorObjects = Union[BaseException, Error]
+ExceptionOrError = Union[BaseException, Error]
 
 
 @dataclass
@@ -68,7 +73,11 @@ class RunResult:
     # Raw output of cell: last expression
     output: Any
     # Exception raised by cell, if any
-    exception: Optional[ErrorObjects]
+    #
+    # TODO(akshayka): Exceptions and "Errors" (most of which are at parse time
+    # and can't be encountered by the runner) shouldn't be packed into a single
+    # field.
+    exception: Optional[ExceptionOrError]
     # Accumulated output: via imperative mo.output.append()
     accumulated_output: Any = None
 
@@ -77,6 +86,26 @@ class RunResult:
         return self.exception is None
 
 
+def should_show_traceback(
+    exception: Optional[ExceptionOrError],
+) -> bool:
+    if exception is None:
+        return True
+
+    # Stop "errors" aren't actually errors but rather a control
+    # flow mechanism used by mo.stop() to stop execution; as such
+    # a traceback should not be shown for them.
+    if isinstance(exception, MarimoStopError):
+        return False
+
+    # SQL parsing errors happen in SQL cells so showing a
+    # python traceback is not useful.
+    if isinstance(exception, MarimoSQLError):
+        return False
+
+    return True
+
+
 class Runner:
     """Runner for a collection of cells."""
 
@@ -142,7 +171,7 @@ class Runner:
         # whether the runner has been interrupted
         self.interrupted = False
         # mapping from cell_id to exception it raised
-        self.exceptions: dict[CellId_t, ErrorObjects] = {}
+        self.exceptions: dict[CellId_t, ExceptionOrError] = {}
 
         # each cell's position in the run queue
         self._run_position = {
@@ -329,7 +358,7 @@ class Runner:
         unwrapped_exception: Optional[BaseException],
         cell_id: CellId_t,
     ) -> tuple[RunResult, Optional[BaseException]]:
-        exception: Optional[ErrorObjects] = unwrapped_exception
+        exception: Optional[ExceptionOrError] = unwrapped_exception
         if isinstance(exception, MarimoMissingRefError):
             ref, blamed_cell = self._get_blamed_cell(exception)
             # All MarimoMissingRefErrors should be caused caused by
@@ -403,6 +432,13 @@ class Runner:
                         exception = output
                 except Exception:
                     pass
+        # Handle SQL parsing errors
+        elif unwrapped_exception is not None and is_sql_parse_error(
+            unwrapped_exception
+        ):
+            cell = self.graph.cells[cell_id]
+            output = create_sql_error_from_exception(unwrapped_exception, cell)
+            exception = output
         elif isinstance(unwrapped_exception, MarimoStopError):
             output = unwrapped_exception.output
             exception = unwrapped_exception
@@ -502,10 +538,7 @@ class Runner:
             # this call as well, so this should be lifted out of `run`.
             self.cancel(cell_id)
 
-            # Stop "errors" aren't actually errors but rather a control
-            # flow mechanism used by mo.stop() to stop execution; as such
-            # a traceback should not be shown for them.
-            if not isinstance(run_result.exception, MarimoStopError):
+            if should_show_traceback(run_result.exception):
                 tmpio = io.StringIO()
                 # The executors explicitly raise cell exceptions from base
                 # exceptions such that the stack trace is cleaner.
