@@ -10,16 +10,18 @@ from marimo._server.models.completion import (
     SchemaTable,
     VariableContext,
 )
+from marimo._types.ids import SessionId
 
 FIM_PREFIX_TAG = "<|fim_prefix|>"
 FIM_SUFFIX_TAG = "<|fim_suffix|>"
 FIM_MIDDLE_TAG = "<|fim_middle|>"
 
-language_rules = {
+LANGUAGES: list[Language] = ["python", "sql", "markdown"]
+language_rules: dict[Language, list[str]] = {
     "python": [
         "For matplotlib: use plt.gca() as the last expression instead of plt.show().",
         "For plotly: return the figure object directly.",
-        "For altair: return the chart object directly. Add tooltips where appropriate.",
+        "For altair: return the chart object directly. Add tooltips where appropriate. You can pass polars dataframes directly to altair (e.g., alt.Chart(df)).",
         "Include proper labels, titles, and color schemes.",
         "Make visualizations interactive where appropriate.",
         "If an import already exists, do not import it again.",
@@ -32,6 +34,15 @@ language_rules = {
 }
 
 
+language_rules_multiple_cells: dict[Language, list[str]] = {
+    "sql": [
+        'SQL cells start with df = mo.sql(f"""<your query>""") for DuckDB, or df = mo.sql(f"""<your query>""", engine=engine) for other SQL engines. You should always write queries inline as the code snippet above, do not use variables to store queries.',
+        "This will automatically display the result in the UI. You do not need to return the dataframe in the cell.",
+        "The SQL must use the syntax of the database engine specified in the `engine` variable. If no engine, then use duckdb syntax.",
+    ]
+}
+
+
 def _format_schema_info(tables: Optional[list[SchemaTable]]) -> str:
     """Helper to format schema information from context"""
     if not tables:
@@ -165,10 +176,13 @@ def get_refactor_or_insert_notebook_cell_system_prompt(
 
     if support_multiple_cells:
         # Add all language rules for multi-cell scenarios
-        for lang in language_rules:
-            if len(language_rules[lang]) > 0:
+        for lang in LANGUAGES:
+            language_rule = language_rules_multiple_cells.get(
+                lang, language_rules.get(lang, [])
+            )
+            if language_rule:
                 system_prompt += (
-                    f"\n\n## Rules for {lang}:\n{_rules(language_rules[lang])}"
+                    f"\n\n## Rules for {lang}:\n{_rules(language_rule)}"
                 )
     elif language in language_rules and language_rules[language]:
         system_prompt += (
@@ -191,7 +205,7 @@ def get_refactor_or_insert_notebook_cell_system_prompt(
     if support_multiple_cells:
         system_prompt += "\n\nAgain, just output code wrapped in cells. Each cell is wrapped in backticks with the appropriate language identifier (python, sql, markdown)."
     else:
-        system_prompt += "\n\nAgain, just output the code itself."
+        system_prompt += f"\n\nAgain, just output the code itself and make sure to return the code as just {language}."
 
     return system_prompt
 
@@ -230,6 +244,30 @@ def _get_mode_intro_message(mode: CopilotMode) -> str:
             "- All tool use is strictly read-only. You may not perform write, edit, or execution actions.\n"
             "- You must always explain to the user why you are using a tool before invoking it.\n"
         )
+    elif mode == "agent":
+        return (
+            f"{base_intro}"
+            "You are in agent mode - you have autonomy to resolve the user's query by using the tools provided. Please keep going until the user's query is completely resolved, before ending your turn and yielding back to the user. Only terminate your turn when you are sure that the problem is solved. \n"
+            "\n\n## Agent Mode\n"
+            "- You are encouraged to edit existing cells in the notebook or add new cells.\n"
+            "- You should do the following things after editing the notebook:\n"
+            "\t 1. Use the lint notebook tool to check for errors and lint issues\n"
+            "\t 2. Run stale cells tool to run the code\n"
+            "\t 3. If there are errors in cells you have added, edit the existing cell. Don't add new cells to correct errors.\n"
+            "- If you say you're about to do something, actually do it in the same turn (run the tool call right after).\n"
+            "- Group code into logical cells, eg. functions should be in separate cells and all the calls will be in one cell. When asked for explanations or summaries, use markdown cells with proper formatting.\n\n"
+            "## Capabilities\n"
+            "- You can use a set of read and write tools to gather additional context from the notebook or environment (e.g., searching code, summarizing data, or reading documentation) and to modify the notebook (e.g., adding cells, editing cells, deleting cells).\n"
+            "## Limitations\n"
+            "- You must always explain to the user why you are using a tool before invoking it.\n"
+        )
+
+
+def _get_session_info(session_id: SessionId) -> str:
+    return (
+        f"Current notebook session ID: {session_id}. "
+        "Use this session_id with tools that require it."
+    )
 
 
 def get_chat_system_prompt(
@@ -238,9 +276,11 @@ def get_chat_system_prompt(
     context: Optional[AiCompletionContext],
     include_other_code: str,
     mode: CopilotMode,
+    session_id: SessionId,
 ) -> str:
     system_prompt: str = f"""
 {_get_mode_intro_message(mode)}
+{_get_session_info(session_id)}
 
 Your goal is to do one of the following two things:
 
@@ -276,6 +316,14 @@ Marimo's reactivity means:
 
 ## Best Practices
 
+<data_handling>
+- Use polars for data manipulation
+- Implement proper data validation
+- Handle missing values appropriately
+- Use efficient data structures
+- A variable in the last expression of a cell is automatically displayed as a table
+</data_handling>
+
 <ui_elements>
 - Access UI element values with .value attribute (e.g., slider.value)
 - Create UI elements in one cell and reference them in later cells
@@ -323,7 +371,8 @@ Marimo's reactivity means:
 
 <example title="Basic UI with reactivity">
 import marimo as mo
-import matplotlib.pyplot as plt
+import altair as alt
+import polars as pl
 import numpy as np
 
 # Create a slider and display it
@@ -335,21 +384,41 @@ n_points  # Display the slider
 x = np.random.rand(n_points.value)
 y = np.random.rand(n_points.value)
 
-plt.figure(figsize=(8, 6))
-plt.scatter(x, y, alpha=0.7)
-plt.title(f"Scatter plot with {{n_points.value}} points")
-plt.xlabel("X axis")
-plt.ylabel("Y axis")
-plt.gca()  # Return the current axes to display the plot
+df = pl.DataFrame({{"x": x, "y": y}})
+
+chart = alt.Chart(df).mark_circle(opacity=0.7).encode(
+    x=alt.X('x', title='X axis'),
+    y=alt.Y('y', title='Y axis')
+).properties(
+    title=f"Scatter plot with {{n_points.value}} points",
+    width=400,
+    height=300
+)
+
+chart
 </example>"""
 
-    for language in language_rules:
-        if len(language_rules[language]) == 0:
-            continue
+    if mode == "agent":
+        # In agent-mode, we can add how to insert cells into the notebook.
+        for lang in LANGUAGES:
+            # check if multiple_cells rules are present for this language
+            rule_to_add = language_rules_multiple_cells.get(
+                lang, language_rules.get(lang, [])
+            )
+            if rule_to_add:
+                system_prompt += (
+                    f"\n\n## Rules for {lang}:\n{_rules(rule_to_add)}"
+                )
 
-        system_prompt += (
-            f"\n\n## Rules for {language}:\n{_rules(language_rules[language])}"
-        )
+        system_prompt += "\n\n## Rules for inserting cells:\n"
+        system_prompt += 'For markdown cells, use `mo.md(f"""{content}""")`\n'
+        system_prompt += 'For sql cells, use `mo.sql(f"""{content}""")`. If a database engine is specified, use `mo.sql(f"""{content}""", engine=engine)` instead.\n'
+    else:
+        for language in language_rules:
+            if len(language_rules[language]) == 0:
+                continue
+
+            system_prompt += f"\n\n## Rules for {language}:\n{_rules(language_rules[language])}"
 
     if custom_rules and custom_rules.strip():
         system_prompt += f"\n\n## Additional rules:\n{custom_rules}"
