@@ -17,6 +17,7 @@ from typing import (
     Union,
     cast,
 )
+from urllib.parse import parse_qs, urlparse
 
 from starlette.exceptions import HTTPException
 
@@ -34,9 +35,12 @@ from marimo._ai._types import ChatMessage
 from marimo._dependencies.dependencies import DependencyManager
 from marimo._server.ai.config import AnyProviderConfig
 from marimo._server.ai.ids import AiModelId
+from marimo._server.ai.tools.types import ToolDefinition
 from marimo._server.api.status import HTTPStatus
 
 TIMEOUT = 30
+# Long-thinking models can take a long time to complete, so we set a longer timeout
+LONG_THINKING_TIMEOUT = 120
 
 if TYPE_CHECKING:
     from anthropic import (  # type: ignore[import-not-found]
@@ -77,10 +81,16 @@ FinishReason = Literal["tool_calls", "stop"]
 # Types for extract_content method return
 DictContent = tuple[
     dict[str, Any],
-    Literal["tool_call_start", "tool_call_end", "reasoning_signature"],
+    Literal[
+        "tool_call_start",
+        "tool_call_end",
+        "reasoning_signature",
+        "tool_call_delta",
+    ],
 ]
-TextContent = tuple[str, Literal["text", "reasoning", "tool_call_delta"]]
+TextContent = tuple[str, Literal["text", "reasoning"]]
 ExtractedContent = Union[TextContent, DictContent]
+ExtractedContentList = list[ExtractedContent]
 
 # Types for format_stream method parameter
 FinishContent = tuple[FinishReason, Literal["finish_reason"]]
@@ -106,6 +116,13 @@ class StreamOptions:
     format_stream: bool = False
 
 
+@dataclass
+class ActiveToolCall:
+    tool_call_id: str
+    tool_call_name: str
+    tool_call_args: str
+
+
 class CompletionProvider(Generic[ResponseT, StreamT], ABC):
     """Base class for AI completion providers."""
 
@@ -119,14 +136,15 @@ class CompletionProvider(Generic[ResponseT, StreamT], ABC):
         messages: list[ChatMessage],
         system_prompt: str,
         max_tokens: int,
+        additional_tools: list[ToolDefinition],
     ) -> StreamT:
         """Create a completion stream."""
         pass
 
     @abstractmethod
     def extract_content(
-        self, response: ResponseT, tool_call_id: Optional[str] = None
-    ) -> Optional[ExtractedContent]:
+        self, response: ResponseT, tool_call_ids: Optional[list[str]] = None
+    ) -> Optional[ExtractedContentList]:
         """Extract content from a response chunk."""
         pass
 
@@ -230,10 +248,9 @@ class CompletionProvider(Generic[ResponseT, StreamT], ABC):
         options = options or StreamOptions()
 
         # Tool info collected from the first chunk
-        tool_call_id: Optional[str] = None
-        tool_call_name: Optional[str] = None
-        # Tool args collected from the tool_call_delta chunks
-        tool_call_args: str = ""
+        tool_calls: dict[str, ActiveToolCall] = {}
+        tool_calls_order: list[str] = []
+
         # Finish reason collected from the last chunk
         finish_reason: Optional[FinishReason] = None
 
@@ -249,99 +266,128 @@ class CompletionProvider(Generic[ResponseT, StreamT], ABC):
             # If we check content first, these chunks get skipped and finish reason is never detected
             finish_reason = self.get_finish_reason(chunk) or finish_reason
 
-            content = self.extract_content(chunk, tool_call_id)
+            content = self.extract_content(chunk, tool_calls_order)
             if not content:
                 continue
 
-            content_data, content_type = content
+            # Loop through all content chunks
+            for content_data, content_type in content:
+                if options.text_only and content_type != "text":
+                    continue
 
-            if options.text_only and content_type != "text":
-                continue
+                # Handle text content with start/delta/end pattern
+                if (
+                    content_type == "text"
+                    and isinstance(content_data, str)
+                    and options.format_stream
+                ):
+                    if not has_text_started:
+                        # Emit text-start event
+                        current_text_id = f"text_{uuid.uuid4().hex}"
+                        yield convert_to_ai_sdk_messages(
+                            "", "text_start", current_text_id
+                        )
+                        has_text_started = True
 
-            # Handle text content with start/delta/end pattern
-            if (
-                content_type == "text"
-                and isinstance(content_data, str)
-                and options.format_stream
-            ):
-                if not has_text_started:
-                    # Emit text-start event
-                    current_text_id = f"text_{uuid.uuid4().hex}"
+                    # Emit text-delta event with the actual content
                     yield convert_to_ai_sdk_messages(
-                        "", "text_start", current_text_id
+                        content_data, "text", current_text_id
                     )
-                    has_text_started = True
+                    continue
 
-                # Emit text-delta event with the actual content
-                yield convert_to_ai_sdk_messages(
-                    content_data, "text", current_text_id
-                )
-                continue
+                # Handle reasoning content with start/delta/end pattern
+                elif (
+                    content_type == "reasoning"
+                    and isinstance(content_data, str)
+                    and options.format_stream
+                ):
+                    if not has_reasoning_started:
+                        # Emit reasoning-start event
+                        current_reasoning_id = f"reasoning_{uuid.uuid4().hex}"
+                        yield convert_to_ai_sdk_messages(
+                            "", "reasoning_start", current_reasoning_id
+                        )
+                        has_reasoning_started = True
 
-            # Handle reasoning content with start/delta/end pattern
-            elif (
-                content_type == "reasoning"
-                and isinstance(content_data, str)
-                and options.format_stream
-            ):
-                if not has_reasoning_started:
-                    # Emit reasoning-start event
-                    current_reasoning_id = f"reasoning_{uuid.uuid4().hex}"
+                    # Emit reasoning-delta event with the actual content
                     yield convert_to_ai_sdk_messages(
-                        "", "reasoning_start", current_reasoning_id
+                        content_data, "reasoning", current_reasoning_id
                     )
-                    has_reasoning_started = True
+                    continue
 
-                # Emit reasoning-delta event with the actual content
-                yield convert_to_ai_sdk_messages(
-                    content_data, "reasoning", current_reasoning_id
-                )
-                continue
+                # Tool handling
+                if content_type == "tool_call_start" and isinstance(
+                    content_data, dict
+                ):
+                    tool_call_id: Optional[str] = content_data.get(
+                        "toolCallId", None
+                    )
+                    tool_call_name: Optional[str] = content_data.get(
+                        "toolName", None
+                    )
+                    # Sometimes GoogleProvider emits the args in the tool_call_start chunk
+                    tool_call_args: str = ""
+                    if content_data.get("args"):
+                        # don't yield args in tool_call_start chunk
+                        # it will throw an error in ai-sdk-ui
+                        tool_call_args = content_data.pop("args")
 
-            # Tool handling
-            if content_type == "tool_call_start" and isinstance(
-                content_data, dict
-            ):
-                tool_call_id = content_data.get("toolCallId", None)
-                tool_call_name = content_data.get("toolName", None)
-                # Sometimes GoogleProvider emits the args in the tool_call_start chunk
-                if content_data.get("args"):
-                    # don't yield args in tool_call_start chunk
-                    # it will throw an error in ai-sdk-ui
-                    tool_call_args = content_data.pop("args")
+                    if tool_call_id and tool_call_name:
+                        # Add new tool calls to the list for tracking
+                        tool_calls_order.append(tool_call_id)
+                        tool_calls[tool_call_id] = ActiveToolCall(
+                            tool_call_id=tool_call_id,
+                            tool_call_name=tool_call_name,
+                            tool_call_args=tool_call_args,
+                        )
 
-            if content_type == "tool_call_delta" and isinstance(
-                content_data, str
-            ):
-                if isinstance(self, GoogleProvider):
-                    # For GoogleProvider, each chunk contains the full (possibly updated) args dict as a JSON string.
-                    # Example: first chunk: {"location": "San Francisco"}
-                    #          second chunk: {"location": "San Francisco", "zip": "94107"}
-                    # We overwrite tool_call_args with the latest chunk.
-                    tool_call_args = content_data
-                else:
-                    # For other providers, tool_call_args is built up incrementally from deltas.
-                    tool_call_args += content_data
-                # update tool_call_delta to ai-sdk-ui structure
-                # based on https://ai-sdk.dev/docs/ai-sdk-ui/stream-protocol#tool-call-delta-part
-                content_data = {
-                    "toolCallId": tool_call_id,
-                    "inputTextDelta": content_data,
-                }
+                if content_type == "tool_call_delta" and isinstance(
+                    content_data, dict
+                ):
+                    tool_call_delta_id = content_data.get("toolCallId", None)
+                    tool_call_delta: str = content_data.get(
+                        "inputTextDelta", ""
+                    )
 
-            content_str = self._content_to_string(content_data)
+                    if not tool_call_delta_id:
+                        if not tool_call_delta_id:
+                            LOGGER.error(
+                                f"Tool call id not found for tool call delta: {content_data}"
+                            )
+                        continue
+                    tool_call = tool_calls.get(tool_call_delta_id, None)
+                    if not tool_call:
+                        continue
 
-            if options.format_stream:
-                stream_content = self._create_stream_content(
-                    content_data, content_type
-                )
-                content_str = self.format_stream(stream_content)
+                    if isinstance(self, GoogleProvider):
+                        # For GoogleProvider, each chunk contains the full (possibly updated) args dict as a JSON string.
+                        # Example: first chunk: {"location": "San Francisco"}
+                        #          second chunk: {"location": "San Francisco", "zip": "94107"}
+                        # We overwrite tool_call_args with the latest chunk.
+                        tool_call.tool_call_args = tool_call_delta
+                    else:
+                        # For other providers, tool_call_args is built up incrementally from deltas.
+                        tool_call.tool_call_args += tool_call_delta
+                    # update tool_call_delta to ai-sdk-ui structure
+                    # based on https://ai-sdk.dev/docs/ai-sdk-ui/stream-protocol#tool-call-delta-part
+                    content_data = {
+                        "toolCallId": tool_call.tool_call_id,
+                        "inputTextDelta": tool_call.tool_call_args,
+                    }
 
-            buffer += content_str
-            original_content += content_str
+                content_str = self._content_to_string(content_data)
 
-            yield buffer
-            buffer = ""
+                if options.format_stream:
+                    stream_content = self._create_stream_content(
+                        content_data, content_type
+                    )
+                    content_str = self.format_stream(stream_content)
+
+                buffer += content_str
+                original_content += content_str
+
+                yield buffer
+                buffer = ""
 
         # Emit text-end event if we started a text block
         if has_text_started and current_text_id and options.format_stream:
@@ -358,19 +404,20 @@ class CompletionProvider(Generic[ResponseT, StreamT], ABC):
             )
 
         # Handle tool call end after the stream is complete
-        if tool_call_id and tool_call_name and not options.text_only:
-            content_data = {
-                "toolCallId": tool_call_id,
-                "toolName": tool_call_name,
-                "input": self.validate_tool_call_args(tool_call_args)
-                or {},  # empty object if tool doesnt have args
-            }
-            content_type = "tool_call_end"
-            yield self.format_stream((content_data, content_type))
-            # Reset tool call state for next stream just in case
-            tool_call_id = None
-            tool_call_name = None
-            tool_call_args = ""
+        if len(tool_calls_order) > 0 and not options.text_only:
+            for tool_call_id in tool_calls_order:
+                tool_call = tool_calls.get(tool_call_id, None)
+                if not tool_call:
+                    continue
+                content_data = {
+                    "toolCallId": tool_call_id,
+                    "toolName": tool_call.tool_call_name,
+                    "input": self.validate_tool_call_args(
+                        tool_call.tool_call_args
+                    ),
+                }
+                content_type = "tool_call_end"
+                yield self.format_stream((content_data, content_type))
 
         # Add a final finish reason chunk
         if finish_reason and not options.text_only:
@@ -392,8 +439,39 @@ class OpenAIProvider(
     DEFAULT_REASONING_EFFORT = "medium"
 
     def _is_reasoning_model(self, model: str) -> bool:
-        # only o-series models support reasoning
-        return model.startswith("o") or model.startswith("gpt-5")
+        """
+        Check if reasoning_effort should be added to the request.
+        Only add for actual OpenAI reasoning models, not for OpenAI-compatible APIs.
+
+        OpenAI-compatible APIs (identified by custom base_url) may not support
+        the reasoning_effort parameter even if the model name suggests it's a
+        reasoning model.
+        """
+        import re
+
+        # Check for reasoning model patterns: o{digit} or gpt-5, with optional openai/ prefix
+        reasoning_patterns = [
+            r"^openai/o\d",  # openai/o1, openai/o3, etc.
+            r"^o\d",  # o1, o3, etc.
+            r"^openai/gpt-5",  # openai/gpt-5*
+            r"^gpt-5",  # gpt-5*
+        ]
+
+        is_reasoning_model_name = any(
+            re.match(pattern, model) for pattern in reasoning_patterns
+        )
+
+        if not is_reasoning_model_name:
+            return False
+
+        # If using a custom base_url that's not OpenAI, don't assume reasoning is supported
+        if (
+            self.config.base_url
+            and "api.openai.com" not in self.config.base_url
+        ):
+            return False
+
+        return True
 
     def get_client(self, config: AnyProviderConfig) -> AsyncOpenAI:
         DependencyManager.openai.require(why="for AI assistance with OpenAI")
@@ -476,6 +554,7 @@ class OpenAIProvider(
         messages: list[ChatMessage],
         system_prompt: str,
         max_tokens: int,
+        additional_tools: list[ToolDefinition],
     ) -> OpenAiStream[ChatCompletionChunk]:
         client = self.get_client(self.config)
         tools = self.config.tools
@@ -491,10 +570,13 @@ class OpenAIProvider(
                 ),
             ),
             "stream": True,
-            "timeout": TIMEOUT,
+            "timeout": LONG_THINKING_TIMEOUT
+            if self._is_reasoning_model(self.model)
+            else TIMEOUT,
         }
         if tools:
-            create_params["tools"] = convert_to_openai_tools(tools)
+            all_tools = tools + additional_tools
+            create_params["tools"] = convert_to_openai_tools(all_tools)
         if self._is_reasoning_model(self.model):
             create_params["reasoning_effort"] = self.DEFAULT_REASONING_EFFORT
             create_params["max_completion_tokens"] = max_tokens
@@ -508,9 +590,9 @@ class OpenAIProvider(
     def extract_content(
         self,
         response: ChatCompletionChunk,
-        tool_call_id: Optional[str] = None,
-    ) -> Optional[ExtractedContent]:
-        del tool_call_id
+        tool_call_ids: Optional[list[str]] = None,
+    ) -> Optional[ExtractedContentList]:
+        tool_call_ids = tool_call_ids or []
         if (
             hasattr(response, "choices")
             and response.choices
@@ -521,29 +603,42 @@ class OpenAIProvider(
             # Text content
             content = delta.content
             if content:
-                return (content, "text")
+                return [(content, "text")]
 
             # Tool call:
             if delta.tool_calls:
-                tool_calls = delta.tool_calls[0]
+                tool_content: ExtractedContentList = []
+                for tool_call in delta.tool_calls:
+                    tool_index = tool_call.index
 
-                # Start of tool call
-                # id is only present for the first tool call chunk
-                if (
-                    tool_calls.id
-                    and tool_calls.function
-                    and tool_calls.function.name
-                ):
-                    tool_info = {
-                        "toolCallId": tool_calls.id,
-                        "toolName": tool_calls.function.name,
-                    }
-                    return (tool_info, "tool_call_start")
+                    # Start of tool call
+                    # id is only present for the first tool call chunk
+                    if (
+                        tool_call.id
+                        and tool_call.function
+                        and tool_call.function.name
+                    ):
+                        tool_info = {
+                            "toolCallId": tool_call.id,
+                            "toolName": tool_call.function.name,
+                        }
+                        tool_content.append((tool_info, "tool_call_start"))
 
-                # Delta of tool call
-                # arguments is only present second chunk onwards
-                if tool_calls.function and tool_calls.function.arguments:
-                    return (tool_calls.function.arguments, "tool_call_delta")
+                    # Delta of tool call
+                    # arguments is only present second chunk onwards
+                    if (
+                        tool_call.function
+                        and tool_call.function.arguments
+                        and tool_call_ids[tool_index]
+                    ):
+                        tool_delta = {
+                            "toolCallId": tool_call_ids[tool_index],
+                            "inputTextDelta": tool_call.function.arguments,
+                        }
+                        tool_content.append((tool_delta, "tool_call_delta"))
+
+                # return the tool content
+                return tool_content
 
         return None
 
@@ -579,24 +674,68 @@ class OpenAIProvider(
 
 
 class AzureOpenAIProvider(OpenAIProvider):
-    def get_client(self, config: AnyProviderConfig) -> AsyncOpenAI:
-        from urllib.parse import parse_qs, urlparse
+    def _is_reasoning_model(self, model: str) -> bool:
+        # https://learn.microsoft.com/en-us/answers/questions/5519548/does-gpt-5-via-azure-support-reasoning-effort-and
+        # Only custom models support reasoning effort, we can expose this as a parameter in the future
+        del model
+        return False
 
+    def _handle_azure_openai(self, base_url: str) -> tuple[str, str, str]:
+        """Handle Azure OpenAI.
+        Sample base URL: https://<your-resource-name>.openai.azure.com/openai/deployments/<deployment_name>?api-version=<api-version>
+
+        Args:
+            base_url (str): The base URL of the Azure OpenAI.
+
+        Returns:
+            tuple[str, str, str]: The API version, deployment name, and endpoint.
+        """
+
+        parsed_url = urlparse(base_url)
+
+        deployment_name = parsed_url.path.split("/")[3]
+        api_version = parse_qs(parsed_url.query)["api-version"][0]
+
+        endpoint = f"{parsed_url.scheme}://{parsed_url.hostname}"
+        return api_version, deployment_name, endpoint
+
+    def get_client(self, config: AnyProviderConfig) -> AsyncOpenAI:
         from openai import AsyncAzureOpenAI
 
         base_url = config.base_url or None
         key = config.api_key
 
-        # Azure OpenAI clients are instantiated slightly differently
-        parsed_url = urlparse(base_url)
-        deployment_model = cast(str, parsed_url.path).split("/")[3]
-        api_version = parse_qs(cast(str, parsed_url.query))["api-version"][0]
+        if base_url is None:
+            raise HTTPException(
+                status_code=HTTPStatus.BAD_REQUEST,
+                detail="Base URL needed to get the endpoint",
+            )
+
+        api_version = None
+        deployment_name = None
+        endpoint = None
+
+        if base_url:
+            if "services.ai.azure.com" in base_url:
+                raise HTTPException(
+                    status_code=HTTPStatus.BAD_REQUEST,
+                    detail="To use Azure AI Foundry, use the OpenAI-compatible provider instead.",
+                )
+            elif "openai.azure.com" in base_url:
+                api_version, deployment_name, endpoint = (
+                    self._handle_azure_openai(base_url)
+                )
+            else:
+                LOGGER.warning(f"Unknown Azure OpenAI base URL: {base_url}")
+                api_version, deployment_name, endpoint = (
+                    self._handle_azure_openai(base_url)
+                )
 
         return AsyncAzureOpenAI(
             api_key=key,
             api_version=api_version,
-            azure_deployment=deployment_model,
-            azure_endpoint=f"{cast(str, parsed_url.scheme)}://{cast(str, parsed_url.hostname)}",
+            azure_deployment=deployment_name,
+            azure_endpoint=endpoint or "",
         )
 
 
@@ -622,6 +761,9 @@ class AnthropicProvider(
     # 1024 tokens is the minimum budget for extended thinking
     DEFAULT_EXTENDED_THINKING_BUDGET_TOKENS = 1024
 
+    # Map of block index to tool call id for tool call delta chunks
+    block_index_to_tool_call_id_map: dict[int, str] = {}
+
     def is_extended_thinking_model(self, model: str) -> bool:
         return any(
             model.startswith(prefix)
@@ -643,11 +785,15 @@ class AnthropicProvider(
 
         return AsyncClient(api_key=config.api_key)
 
+    def maybe_get_tool_call_id(self, block_index: int) -> Optional[str]:
+        return self.block_index_to_tool_call_id_map.get(block_index, None)
+
     async def stream_completion(
         self,
         messages: list[ChatMessage],
         system_prompt: str,
         max_tokens: int,
+        additional_tools: list[ToolDefinition],
     ) -> AnthropicStream[RawMessageStreamEvent]:
         client = self.get_client(self.config)
         tools = self.config.tools
@@ -663,7 +809,8 @@ class AnthropicProvider(
             "temperature": self.get_temperature(),
         }
         if tools:
-            create_params["tools"] = convert_to_anthropic_tools(tools)
+            all_tools = tools + additional_tools
+            create_params["tools"] = convert_to_anthropic_tools(all_tools)
         if self.is_extended_thinking_model(self.model):
             create_params["thinking"] = {
                 "type": "enabled",
@@ -674,12 +821,15 @@ class AnthropicProvider(
             await client.messages.create(**create_params),
         )
 
+    def block_index_to_tool_call_id(self, block_index: int) -> str:
+        return f"tool_call_{block_index}"
+
     def extract_content(
         self,
         response: RawMessageStreamEvent,
-        tool_call_id: Optional[str] = None,
-    ) -> Optional[ExtractedContent]:
-        del tool_call_id
+        tool_call_ids: Optional[list[str]] = None,
+    ) -> Optional[ExtractedContentList]:
+        del tool_call_ids
         from anthropic.types import (
             InputJSONDelta,
             RawContentBlockDeltaEvent,
@@ -693,25 +843,46 @@ class AnthropicProvider(
         # For streaming content
         if isinstance(response, RawContentBlockDeltaEvent):
             if isinstance(response.delta, TextDelta):
-                return (response.delta.text, "text")
+                return [(response.delta.text, "text")]
             if isinstance(response.delta, ThinkingDelta):
-                return (response.delta.thinking, "reasoning")
+                return [(response.delta.thinking, "reasoning")]
             if isinstance(response.delta, InputJSONDelta):
-                return (response.delta.partial_json, "tool_call_delta")
+                block_index = response.index
+                tool_call_id = self.maybe_get_tool_call_id(block_index)
+                if not tool_call_id:
+                    LOGGER.error(
+                        f"Tool call id not found for block index: {response.index}"
+                    )
+                    return None
+                delta_json = response.delta.partial_json
+                tool_delta = {
+                    "toolCallId": tool_call_id,
+                    "inputTextDelta": delta_json,
+                }
+                return [(tool_delta, "tool_call_delta")]
             if isinstance(response.delta, SignatureDelta):
-                return (
-                    {"signature": response.delta.signature},
-                    "reasoning_signature",
-                )
+                return [
+                    (
+                        {"signature": response.delta.signature},
+                        "reasoning_signature",
+                    )
+                ]
 
         # For the beginning of a tool use block
         if isinstance(response, RawContentBlockStartEvent):
             if isinstance(response.content_block, ToolUseBlock):
+                tool_call_id = response.content_block.id
+                tool_call_name = response.content_block.name
+                block_index = response.index
+                # Store the tool call id for the block index
+                self.block_index_to_tool_call_id_map[block_index] = (
+                    tool_call_id
+                )
                 tool_info = {
-                    "toolCallId": response.content_block.id,
-                    "toolName": response.content_block.name,
+                    "toolCallId": tool_call_id,
+                    "toolName": tool_call_name,
                 }
-                return (tool_info, "tool_call_start")
+                return [(tool_info, "tool_call_start")]
 
         return None
 
@@ -746,13 +917,19 @@ class GoogleProvider(
         "gemini-2.5-flash",
     ]
 
+    # Keep a persistent async client to avoid closing during stream iteration
+    _client: Optional[GoogleClient] = None
+
     def is_thinking_model(self, model: str) -> bool:
         return any(
             model.startswith(prefix) for prefix in self.THINKING_MODEL_PREFIXES
         )
 
     def get_config(
-        self, system_prompt: str, max_tokens: int
+        self,
+        system_prompt: str,
+        max_tokens: int,
+        additional_tools: list[ToolDefinition],
     ) -> GenerateContentConfig:
         tools = self.config.tools
         config = {
@@ -761,7 +938,8 @@ class GoogleProvider(
             "max_output_tokens": max_tokens,
         }
         if tools:
-            config["tools"] = convert_to_google_tools(tools)
+            all_tools = tools + additional_tools
+            config["tools"] = convert_to_google_tools(all_tools)
         if self.is_thinking_model(self.model):
             config["thinking_config"] = {
                 "include_thoughts": True,
@@ -777,24 +955,52 @@ class GoogleProvider(
             )
             from google import genai  # type: ignore
 
-        return genai.Client(api_key=config.api_key).aio
+        # Reuse a stored async client if already created
+        if self._client is not None:
+            return self._client
+
+        # If no API key is provided, try to use environment variables and ADC
+        # This supports Google Vertex AI usage without explicit API keys
+        if not config.api_key:
+            # Check if GOOGLE_GENAI_USE_VERTEXAI is set to enable Vertex AI mode
+            use_vertex = (
+                os.getenv("GOOGLE_GENAI_USE_VERTEXAI", "").lower() == "true"
+            )
+            if use_vertex:
+                project = os.getenv("GOOGLE_CLOUD_PROJECT")
+                location = os.getenv("GOOGLE_CLOUD_LOCATION", "us-central1")
+                self._client = genai.Client(
+                    vertexai=True, project=project, location=location
+                ).aio
+            else:
+                # Try default initialization which may work with environment variables
+                self._client = genai.Client().aio
+
+            # Return vertex or default client
+            return self._client
+
+        self._client = genai.Client(api_key=config.api_key).aio
+        return self._client
 
     async def stream_completion(
         self,
         messages: list[ChatMessage],
         system_prompt: str,
         max_tokens: int,
+        additional_tools: list[ToolDefinition],
     ) -> AsyncIterator[GenerateContentResponse]:
         client = self.get_client(self.config)
-        return await client.models.generate_content_stream(
+        return await client.models.generate_content_stream(  # type: ignore[reportReturnType]
             model=self.model,
             contents=convert_to_google_messages(messages),
             config=self.get_config(
-                system_prompt=system_prompt, max_tokens=max_tokens
+                system_prompt=system_prompt,
+                max_tokens=max_tokens,
+                additional_tools=additional_tools,
             ),
         )
 
-    def _get_tool_call_id(self, tool_call_id: Optional[str]) -> Optional[str]:
+    def _get_tool_call_id(self, tool_call_id: Optional[str]) -> str:
         # Custom tools don't have an id, so we have to generate a random uuid
         # https://ai.google.dev/gemini-api/docs/function-calling?example=meeting
         if not tool_call_id:
@@ -805,8 +1011,9 @@ class GoogleProvider(
     def extract_content(
         self,
         response: GenerateContentResponse,
-        tool_call_id: Optional[str] = None,
-    ) -> Optional[ExtractedContent]:
+        tool_call_ids: Optional[list[str]] = None,
+    ) -> Optional[ExtractedContentList]:
+        tool_call_ids = tool_call_ids or []
         if not response.candidates:
             return None
 
@@ -817,33 +1024,56 @@ class GoogleProvider(
         if not candidate.content.parts:
             return None
 
-        for part in candidate.content.parts:
-            # Start of tool call
-            # GoogleProvider may emit the function_call object in every chunk, not just the first.
-            # We use tool_call_id to ensure we only emit one tool_call_start event per tool call.
-            if part.function_call and not tool_call_id:
-                tool_info = {
-                    "toolCallId": self._get_tool_call_id(
-                        part.function_call.id
-                    ),
-                    "toolName": part.function_call.name,
-                    "args": json.dumps(part.function_call.args),
-                }
-                return (tool_info, "tool_call_start")
-            # Tool call args (not delta)
-            elif part.function_call and part.function_call.args:
-                return (json.dumps(part.function_call.args), "tool_call_delta")
+        # Build events by first scanning parts and rectifying tool calls by position
+        content: ExtractedContentList = []
+        function_call_index = -1
+        seen_in_frame: set[int] = set()
 
-            # Skip non-text content
-            elif part.text:
-                # Reasoning content
-                if part.thought:
-                    return (part.text, "reasoning")
+        for part in candidate.content.parts:
+            # Handle function calls (may appear multiple times per chunk)
+            if part.function_call:
+                function_call_index += 1
+                # Resolve a stable id by position if provided from the caller; else synthesize
+                stable_id = (
+                    tool_call_ids[function_call_index]
+                    if function_call_index < len(tool_call_ids)
+                    and tool_call_ids[function_call_index]
+                    else self._get_tool_call_id(part.function_call.id)
+                )
+
+                # First sight of this call index in this frame => emit start
+                if function_call_index not in seen_in_frame:
+                    tool_info = {
+                        "toolCallId": stable_id,
+                        "toolName": part.function_call.name,
+                        "args": json.dumps(part.function_call.args),
+                    }
+                    content.append((tool_info, "tool_call_start"))
+                    seen_in_frame.add(function_call_index)
                 else:
-                    return (part.text, "text")
-            else:
+                    # Subsequent occurrences for the same index => treat as delta (snapshot semantics)
+                    if part.function_call.args is not None:
+                        tool_delta = {
+                            "toolCallId": stable_id,
+                            "inputTextDelta": json.dumps(
+                                part.function_call.args
+                            ),
+                        }
+                        content.append((tool_delta, "tool_call_delta"))
                 continue
-        return None
+
+            # Text/Reasoning handling
+            if part.text:
+                if part.thought:
+                    content.append((part.text, "reasoning"))
+                else:
+                    content.append((part.text, "text"))
+                continue
+
+            # Ignore other non-text parts (e.g., images) at this layer
+            continue
+
+        return content
 
     def get_finish_reason(
         self, response: GenerateContentResponse
@@ -890,6 +1120,7 @@ class BedrockProvider(
         messages: list[ChatMessage],
         system_prompt: str,
         max_tokens: int,
+        additional_tools: list[ToolDefinition],
     ) -> LitellmStream:
         DependencyManager.litellm.require(why="for AI assistance with Bedrock")
         DependencyManager.boto3.require(why="for AI assistance with Bedrock")
@@ -912,16 +1143,17 @@ class BedrockProvider(
             "timeout": TIMEOUT,
         }
         if tools:
-            config["tools"] = convert_to_openai_tools(tools)
+            all_tools = tools + additional_tools
+            config["tools"] = convert_to_openai_tools(all_tools)
 
         return await litellm_completion(**config)
 
     def extract_content(
         self,
         response: LitellmStreamResponse,
-        tool_call_id: Optional[str] = None,
-    ) -> Optional[ExtractedContent]:
-        del tool_call_id
+        tool_call_ids: Optional[list[str]] = None,
+    ) -> Optional[ExtractedContentList]:
+        tool_call_ids = tool_call_ids or []
         if (
             hasattr(response, "choices")
             and response.choices
@@ -932,30 +1164,42 @@ class BedrockProvider(
             # Text content
             content = delta.content
             if content:
-                return (str(content), "text")
+                return [(str(content), "text")]
 
             # Tool call: LiteLLM follows OpenAI format for tool calls
+
             if hasattr(delta, "tool_calls") and delta.tool_calls:
-                tool_calls = delta.tool_calls[0]
+                tool_content: ExtractedContentList = []
 
-                # Start of tool call
-                # id is only present for the first tool call chunk
-                if hasattr(tool_calls, "id") and tool_calls.id:
-                    tool_info = {
-                        "toolCallId": tool_calls.id,
-                        "toolName": tool_calls.function.name,
-                    }
-                    return (tool_info, "tool_call_start")
+                for tool_call in delta.tool_calls:
+                    tool_index: int = tool_call.index
 
-                # Delta of tool call
-                # arguments is only present second chunk onwards
-                if (
-                    hasattr(tool_calls, "function")
-                    and tool_calls.function
-                    and hasattr(tool_calls.function, "arguments")
-                    and tool_calls.function.arguments
-                ):
-                    return (tool_calls.function.arguments, "tool_call_delta")
+                    # Start of tool call
+                    # id is only present for the first tool call chunk
+                    if hasattr(tool_call, "id") and tool_call.id:
+                        tool_info = {
+                            "toolCallId": tool_call.id,
+                            "toolName": tool_call.function.name,
+                        }
+                        tool_content.append((tool_info, "tool_call_start"))
+
+                    # Delta of tool call
+                    # arguments is only present second chunk onwards
+                    if (
+                        hasattr(tool_call, "function")
+                        and tool_call.function
+                        and hasattr(tool_call.function, "arguments")
+                        and tool_call.function.arguments
+                        and tool_call_ids[tool_index]
+                    ):
+                        tool_delta = {
+                            "toolCallId": tool_call_ids[tool_index],
+                            "inputTextDelta": tool_call.function.arguments,
+                        }
+                        tool_content.append((tool_delta, "tool_call_delta"))
+
+                # return the tool content
+                return tool_content
 
         return None
 
@@ -988,6 +1232,8 @@ def get_completion_provider(
         return BedrockProvider(model_id.model, config)
     elif model_id.provider == "azure":
         return AzureOpenAIProvider(model_id.model, config)
+    elif model_id.provider == "openrouter":
+        return OpenAIProvider(model_id.model, config)
     else:
         return OpenAIProvider(model_id.model, config)
 
