@@ -9,6 +9,7 @@ from starlette.exceptions import HTTPException
 from starlette.responses import HTMLResponse, JSONResponse, PlainTextResponse
 
 from marimo import _loggers
+from marimo._dependencies.dependencies import DependencyManager
 from marimo._messaging.msgspec_encoder import asdict
 from marimo._server.api.deps import AppState
 from marimo._server.api.status import HTTPStatus
@@ -350,6 +351,11 @@ async def auto_export_as_ipynb(
         return PlainTextResponse(status_code=HTTPStatus.NOT_MODIFIED)
 
     async def _background_export() -> None:
+        # Check has nbformat installed
+        if not DependencyManager.nbformat.has():
+            LOGGER.error("Cannot snapshot to IPYNB: nbformat not installed")
+            return
+
         # Reload the file manager to get the latest state
         session.app_file_manager.reload()
 
