@@ -53,6 +53,7 @@ from marimo._server.templates.templates import (
 )
 from marimo._server.tokens import SkewProtectionToken
 from marimo._types.ids import CellId_t
+from marimo._utils import async_path
 from marimo._utils.code import hash_code
 from marimo._utils.data_uri import build_data_url
 from marimo._utils.marimo_path import MarimoPath
@@ -247,7 +248,6 @@ class Exporter:
         filename: Optional[str],
         previous: Path | None = None,
     ) -> tuple[str, str]:
-        from marimo._ast import codegen
         from marimo._ast.app_config import _AppConfig
         from marimo._ast.compiler import compile_cell
         from marimo._convert.markdown.markdown import (
@@ -291,7 +291,7 @@ class Exporter:
                 metadata["header"] = header.strip()
         else:
             header_file = previous if previous else filename
-            if header_file:
+            if header_file and Path(header_file).exists():
                 with open(header_file, encoding="utf-8") as f:
                     _metadata, _ = extract_frontmatter(f.read())
                 metadata.update(_metadata)
@@ -360,6 +360,10 @@ class Exporter:
                         previous_was_markdown = True
                         document.append(markdown)
                         continue
+                    # In which case we need to format it like our python blocks.
+                    elif cell_impl.markdown:
+                        code = codegen.format_markdown(cell_impl)
+
                 attributes["language"] = language
                 # Definitely a code cell, but need to determine if it can be
                 # formatted as non-python.
@@ -529,10 +533,10 @@ class AutoExporter:
         if export_dir in self._created_dirs:
             return
 
-        if not directory.exists():
+        if not await async_path.exists(directory):
             raise FileNotFoundError(f"Directory {directory} does not exist")
 
-        export_dir.mkdir(parents=True, exist_ok=True)
+        await async_path.mkdir(export_dir, parents=True, exist_ok=True)
 
         # Cache that we've created this directory
         self._created_dirs.add(export_dir)
