@@ -50,12 +50,14 @@ from marimo._runtime import requests, runtime
 from marimo._runtime.requests import (
     AppMetadata,
     CreationRequest,
+    DeleteCellRequest,
     ExecuteMultipleRequest,
     ExecutionRequest,
     HTTPRequest,
     SerializedCLIArgs,
     SerializedQueryParams,
     SetUIElementValueRequest,
+    SyncGraphRequest,
 )
 from marimo._server.exceptions import InvalidSessionException
 from marimo._server.file_manager import AppFileManager
@@ -73,6 +75,7 @@ from marimo._server.tokens import AuthToken, SkewProtectionToken
 from marimo._server.types import ProcessLike
 from marimo._server.utils import print_, print_tabbed
 from marimo._types.ids import CellId_t, ConsumerId, SessionId
+from marimo._utils import async_path
 from marimo._utils.disposable import Disposable
 from marimo._utils.distributor import (
     ConnectionDistributor,
@@ -191,7 +194,7 @@ class KernelManager:
     def start_kernel(self) -> None:
         # We use a process in edit mode so that we can interrupt the app
         # with a SIGINT; we don't mind the additional memory consumption,
-        # since there's only one client sess
+        # since there's only one client session
         is_edit_mode = self.mode == SessionMode.EDIT
         listener = None
         if is_edit_mode:
@@ -578,19 +581,25 @@ class Session:
         if isinstance(request, SetUIElementValueRequest):
             self._queue_manager.set_ui_element_queue.put(request)
         # Propagate the control request to the room
-        if isinstance(request, ExecuteMultipleRequest):
+        if isinstance(request, (ExecuteMultipleRequest, SyncGraphRequest)):
+            if isinstance(request, ExecuteMultipleRequest):
+                cell_ids = request.cell_ids
+                codes = request.codes
+            else:
+                cell_ids = request.run_ids
+                codes = [request.cells[cell_id] for cell_id in cell_ids]
             self.room.broadcast(
                 UpdateCellCodes(
-                    cell_ids=request.cell_ids,
-                    codes=request.codes,
+                    cell_ids=cell_ids,
+                    codes=codes,
                     # Not stale because we just ran the code
                     code_is_stale=False,
                 ),
                 except_consumer=from_consumer_id,
             )
-            if len(request.cell_ids) == 1:
+            if len(cell_ids) == 1:
                 self.room.broadcast(
-                    FocusCell(cell_id=request.cell_ids[0]),
+                    FocusCell(cell_id=cell_ids[0]),
                     except_consumer=from_consumer_id,
                 )
         self.session_view.add_control_request(request)
@@ -890,7 +899,7 @@ class SessionManager:
 
         async def on_file_changed(path: Path) -> None:
             # Skip if the session does not relate to the file
-            if session.app_file_manager.path != os.path.abspath(path):
+            if session.app_file_manager.path != await async_path.abspath(path):
                 return
 
             # Use the centralized file change handler
@@ -1140,7 +1149,7 @@ class SessionFileChangeHandler:
         This method reloads the notebook and sends appropriate operations
         to the frontend when a marimo file is modified.
         """
-        abs_file_path = os.path.abspath(file_path)
+        abs_file_path = await async_path.abspath(file_path)
 
         # Use a lock to prevent concurrent processing of the same file
         if abs_file_path not in self._file_change_locks:
@@ -1191,6 +1200,11 @@ class SessionFileChangeHandler:
         # Get the latest codes
         codes = list(session.app_file_manager.app.cell_manager.codes())
         cell_ids = list(session.app_file_manager.app.cell_manager.cell_ids())
+
+        LOGGER.info(
+            f"File changed: {file_path}. num_cell_ids: {len(cell_ids)}, num_codes: {len(codes)}, changed_cell_ids: {changed_cell_ids}"
+        )
+
         # Send the updated cell ids and codes to the frontend
         session.write_operation(
             UpdateCellIdsRequest(cell_ids=cell_ids),
@@ -1202,28 +1216,29 @@ class SessionFileChangeHandler:
             "watcher_on_save"
         ]
         should_autorun = watcher_on_save == "autorun"
+        deleted = {
+            cell_id for cell_id in changed_cell_ids if cell_id not in cell_ids
+        }
 
         # Auto-run cells if configured
         if should_autorun:
-            changed_cell_ids_list = list(changed_cell_ids)
-            cell_ids_to_idx = {
-                cell_id: idx for idx, cell_id in enumerate(cell_ids)
-            }
-            changed_codes = [
-                codes[cell_ids_to_idx[cell_id]]
-                for cell_id in changed_cell_ids_list
-            ]
+            changed_cell_ids_list = list(changed_cell_ids - deleted)
+            cells = dict(zip(cell_ids, codes))
 
-            # This runs the request and also runs UpdateCellCodes
             session.put_control_request(
-                ExecuteMultipleRequest(
-                    cell_ids=changed_cell_ids_list,
-                    codes=changed_codes,
-                    request=None,
+                SyncGraphRequest(
+                    cells=cells,
+                    run_ids=changed_cell_ids_list,
+                    delete_ids=list(deleted),
                 ),
                 from_consumer_id=None,
             )
         else:
+            for to_delete in deleted:
+                session.put_control_request(
+                    DeleteCellRequest(cell_id=to_delete),
+                    from_consumer_id=None,
+                )
             session.write_operation(
                 UpdateCellCodes(
                     cell_ids=cell_ids,
