@@ -1,24 +1,25 @@
 # Copyright 2024 Marimo. All rights reserved.
+
 import marimo
 
-__generated_with = "0.6.22"
+__generated_with = "0.15.5"
 app = marimo.App(width="medium")
 
 
 @app.cell
-def __():
+def _():
     import marimo as mo
-    return mo,
+    return (mo,)
 
 
 @app.cell
-def __(mo):
+def _(mo):
     t = mo.ui.table({"a": [1, 2, 3], "b": [4, 5, 6]})
-    return t,
+    return (t,)
 
 
 @app.cell
-def __(t):
+def _(t):
     t
     return
 
