@@ -9,42 +9,42 @@
 
 import marimo
 
-__generated_with = "0.2.2"
+__generated_with = "0.15.5"
 app = marimo.App(width="full")
 
 
 @app.cell
-def __():
+def _():
     import marimo as mo
     from vega_datasets import data
     return data, mo
 
 
 @app.cell
-def __(data, mo):
+def _(data, mo):
     options = data.list_datasets()
     dataset_dropdown = mo.ui.dropdown(options, label="Datasets", value="cars")
     dataset_dropdown
-    return dataset_dropdown, options
+    return (dataset_dropdown,)
 
 
 @app.cell
-def __(data, dataset_dropdown, mo):
+def _(data, dataset_dropdown, mo):
     mo.stop(not dataset_dropdown.value)
     selected_dataset = dataset_dropdown.value
     df = data.__call__(selected_dataset)
-    return df, selected_dataset
+    return (df,)
 
 
 @app.cell
-def __(df, mo):
+def _(df, mo):
     v = mo.ui.data_explorer(df)
     v
-    return v,
+    return (v,)
 
 
 @app.cell
-def __(v):
+def _(v):
     v.value
     return
 
