@@ -1,6 +1,6 @@
 import marimo
 
-__generated_with = "0.12.8"
+__generated_with = "0.15.5"
 app = marimo.App(width="full")
 
 
@@ -9,7 +9,7 @@ def _():
     import plotly.express as px
     fig = px.scatter(x=[0, 1, 2, 3, 4], y=[0, 1, 4, 9, 16])
     fig.show()
-    return fig, px
+    return
 
 
 if __name__ == "__main__":
