@@ -2,7 +2,7 @@
 
 import marimo
 
-__generated_with = "0.15.5"
+__generated_with = "0.17.4"
 app = marimo.App()
 
 with app.setup:
@@ -18,120 +18,112 @@ def _():
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        # File Format
+    mo.md(r"""
+    # File Format
 
-        marimo apps are stored as pure Python files.
+    marimo apps are stored as pure Python files.
 
-        These files are:
+    These files are:
 
-        - 🤖 legible for both humans and machines
-        - ✏️ formattable using your tool of choice
-        - ➕ easily versioned with git, producing small diffs
-        - 🐍 usable as Python  scripts, with UI  elements taking their default values
-        - 🧩 modular, exposing functions and classes that can be imported from the notebook
-        """
-    )
+    - 🤖 legible for both humans and machines
+    - ✏️ formattable using your tool of choice
+    - ➕ easily versioned with git, producing small diffs
+    - 🐍 usable as Python  scripts, with UI  elements taking their default values
+    - 🧩 modular, exposing functions and classes that can be imported from the notebook
+    """)
     return
 
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        """
-        ## Example
+    mo.md("""
+    ## Example
 
-        Consider a marimo notebook with the following four cells.
+    Consider a marimo notebook with the following four cells.
 
-        First cell:
-        ```python3
+    First cell:
+    ```python3
+    print(text.value)
+    ```
+
+    Second cell:
+    ```python3
+    def say_hello(name="World"):
+        return f"Hello, {name}!"
+    ```
+
+    Third cell:
+    ```python3
+    text = mo.ui.text(value=say_hello())
+    text
+    ```
+
+    Fourth cell:
+    ```python3
+    import marimo as mo
+    ```
+    """)
+    return
+
+
+@app.cell(hide_code=True)
+def _(mo):
+    mo.md("""
+    For the above example, marimo would generate the following file
+    contents:
+
+    ```python3
+    import marimo
+
+    __generated_with = "0.0.0"
+    app = marimo.App()
+
+    @app.cell
+    def _(text):
         print(text.value)
-        ```
+        return
 
-        Second cell:
-        ```python3
-        def say_hello(name="World"):
-            return f"Hello, {name}!"
-        ```
+    @app.function
+    def say_hello(name="World"):
+        return f"Hello, {name}!"
 
-        Third cell:
-        ```python3
+    @app.cell
+    def _(mo):
         text = mo.ui.text(value=say_hello())
         text
-        ```
+        return (text,)
 
-        Fourth cell:
-        ```python3
+    @app.cell
+    def _():
         import marimo as mo
-        ```
-        """
-    )
+        return mo,
+
+    if __name__ == "__main__":
+        app.run()
+    ```
+
+    As you can see, this is _pure Python_. This is part of the reason why
+    marimo's generated files are **git-friendly**: small changes made using
+    the marimo editor result in small changes to the file that marimo
+    generates.
+
+    Moreover, the cell defining a single pure function `say_hello` was saved "top-level" in the notebook file, making it possible for you to import it into other Python files or notebooks.
+    """)
     return
 
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        """
-        For the above example, marimo would generate the following file
-        contents:
+    mo.md("""
+    ## Properties
 
-        ```python3
-        import marimo
+    marimo's file format was designed to be easy to read and easy
+    to work with, while also serving the needs of the marimo library. You can
+    even edit the generated file's cells directly, using your favorite text
+    editor, and format the file with your favorite code formatter.
 
-        __generated_with = "0.0.0"
-        app = marimo.App()
-
-        @app.cell
-        def _(text):
-            print(text.value)
-            return
-
-        @app.function
-        def say_hello(name="World"):
-            return f"Hello, {name}!"
-
-        @app.cell
-        def _(mo):
-            text = mo.ui.text(value=say_hello())
-            text
-            return (text,)
-
-        @app.cell
-        def _():
-            import marimo as mo
-            return mo,
-
-        if __name__ == "__main__":
-            app.run()
-        ```
-
-        As you can see, this is _pure Python_. This is part of the reason why
-        marimo's generated files are **git-friendly**: small changes made using
-        the marimo editor result in small changes to the file that marimo
-        generates.
-
-        Moreover, the cell defining a single pure function `say_hello` was saved "top-level" in the notebook file, making it possible for you to import it into other Python files or notebooks.
-        """
-    )
-    return
-
-
-@app.cell(hide_code=True)
-def _(mo):
-    mo.md(
-        """
-        ## Properties
-
-        marimo's file format was designed to be easy to read and easy
-        to work with, while also serving the needs of the marimo library. You can
-        even edit the generated file's cells directly, using your favorite text
-        editor, and format the file with your favorite code formatter.
-
-        We explain some properties of marimo's file format below.
-        """
-    )
+    We explain some properties of marimo's file format below.
+    """)
     return
 
 
@@ -247,60 +239,54 @@ def _(mo):
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        ## Importing functions and classes from notebooks
+    mo.md(r"""
+    ## Importing functions and classes from notebooks
 
-        The details of marimo's file format are important if you want to import
-        functions and classes defined in your notebook into other Python modules. If you
-        don't intend to do so, you can skip this section.
-        """
-    )
+    The details of marimo's file format are important if you want to import
+    functions and classes defined in your notebook into other Python modules. If you
+    don't intend to do so, you can skip this section.
+    """)
     return
 
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        ### Declaring imports used by functions and classes
+    mo.md(r"""
+    ### Declaring imports used by functions and classes
 
-        marimo can serialize functions and classes into the top-level of a file, so you can import them with regular Python syntax:
+    marimo can serialize functions and classes into the top-level of a file, so you can import them with regular Python syntax:
 
-        ```python
-        from my_notebook import my_function
-        ```
+    ```python
+    from my_notebook import my_function
+    ```
 
-        In particular, if a cell defines just a single function or class, and if that function or class is pure
-        save for references to variables defined in a special **setup cell**, it will be serialized top-level.
+    In particular, if a cell defines just a single function or class, and if that function or class is pure
+    save for references to variables defined in a special **setup cell**, it will be serialized top-level.
 
-        **Setup cell.** Notebooks can optionally include a setup cell that imports modules,
-        written in the file as:
+    **Setup cell.** Notebooks can optionally include a setup cell that imports modules,
+    written in the file as:
 
-        <!-- note this setup cell is hardcoded in the playground example -->
-        ```python
-        with app.setup:
-            import marimo as mo
-            import dataclasses
-        ```
+    <!-- note this setup cell is hardcoded in the playground example -->
+    ```python
+    with app.setup:
+        import marimo as mo
+        import dataclasses
+    ```
 
-        Modules imported in a setup cell can be used in "top-level" functions or
-        classes. You can add the setup cell from the general menu of the editor under:
-        ::lucide:diamond-plus:: Add setup cell.
-        """
-    )
+    Modules imported in a setup cell can be used in "top-level" functions or
+    classes. You can add the setup cell from the general menu of the editor under:
+    ::lucide:diamond-plus:: Add setup cell.
+    """)
     return
 
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        ### Functions and classes
+    mo.md(r"""
+    ### Functions and classes
 
-        Notebook files expose functions and classes that depend only on variables defined in the setup cell (or on other such functions or classes). For example, the following cell:
-        """
-    )
+    Notebook files expose functions and classes that depend only on variables defined in the setup cell (or on other such functions or classes). For example, the following cell:
+    """)
     return
 
 
@@ -316,35 +302,35 @@ def roll_die():
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        ... is saved in the notebook file as
+    mo.md(r"""
+    ... is saved in the notebook file as
 
-        ```python
-        @app.function
-        def roll_die():
-            '''
-            A reusable function.
+    ```python
+    @app.function
+    def roll_die():
+        '''
+        A reusable function.
 
-            Notice the indicator in the bottom right of the cell.
-            '''
-            return random.randint(1, 7)
-        ```
+        Notice the indicator in the bottom right of the cell.
+        '''
+        return random.randint(1, 7)
+    ```
 
 
-        Making it importable as
+    Making it importable as
 
-        ```python
-        from my_notebook import roll_die
-        ```
-        """
-    )
+    ```python
+    from my_notebook import roll_die
+    ```
+    """)
     return
 
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(r"""Standalone classes are also exposed:""")
+    mo.md(r"""
+    Standalone classes are also exposed:
+    """)
     return
 
 
@@ -359,39 +345,35 @@ class SimulationExample:
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        This class is saved in the file as
+    mo.md(r"""
+    This class is saved in the file as
 
-        ```python
-        @app.class_definition
-        @dataclasses.dataclass
-        class SimulationExample:
-            n_rolls: int
+    ```python
+    @app.class_definition
+    @dataclasses.dataclass
+    class SimulationExample:
+        n_rolls: int
 
-            def simulate(self) -> list[int]:
-                return [roll_die() for _ in range(self.n_rolls)]
-        ```
-        """
-    )
+        def simulate(self) -> list[int]:
+            return [roll_die() for _ in range(self.n_rolls)]
+    ```
+    """)
     return
 
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        /// attention | Heads up
-        ///
+    mo.md(r"""
+    /// attention | Heads up
+    ///
 
-        Not all standalone functions will be exposed in the module. If your
-        function depends on variables that are defined in other cells, then it won't
-        be exposed top-level.
+    Not all standalone functions will be exposed in the module. If your
+    function depends on variables that are defined in other cells, then it won't
+    be exposed top-level.
 
 
-        For example, this function will not be exposed:
-        """
-    )
+    For example, this function will not be exposed:
+    """)
     return
 
 
@@ -416,36 +398,32 @@ def wrapped_function_example(variable):
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        ## FAQ
+    mo.md(r"""
+    ## FAQ
 
-        ### I want to edit notebooks in a different editor, what do I need to know?
+    ### I want to edit notebooks in a different editor, what do I need to know?
 
-        See the docs on [using your own editor](https://docs.marimo.io/guides/editor_features/watching/).
+    See the docs on [using your own editor](https://docs.marimo.io/guides/editor_features/watching/).
 
-        ### I want to import functions from a marimo notebook, what do I need to know?
+    ### I want to import functions from a marimo notebook, what do I need to know?
 
-        See the docs on [reusable functions and
-        classes](https://links.marimo.app/reusable-functions).
+    See the docs on [reusable functions and
+    classes](https://links.marimo.app/reusable-functions).
 
-        ### I want to run pytest on marimo notebooks, what do I need to know?
+    ### I want to run pytest on marimo notebooks, what do I need to know?
 
-        See the docs on [testing](https://docs.marimo.io/guides/testing/).
-        """
-    )
+    See the docs on [testing](https://docs.marimo.io/guides/testing/).
+    """)
     return
 
 
 @app.cell(hide_code=True)
 def _(mo):
-    mo.md(
-        r"""
-        ## This notebook's source code
+    mo.md(r"""
+    ## This notebook's source code
 
-        The source code of this notebook is shown below:
-        """
-    )
+    The source code of this notebook is shown below:
+    """)
     return
 
 
